﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using Quartz;
using SCC.Common;
using SCC.Models;
using SCC.Interface;
using HtmlAgilityPack;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;

namespace SCC.Crawler.GP
{
    /// <summary>
    /// 数据爬取类
    /// 安徽11选5
    /// </summary>
    [DisallowConcurrentExecution]
    [PersistJobDataAfterExecution]
    public class AH11X5Job : IJob
    {
        /// <summary>
        /// 构造函数
        /// </summary>
        public AH11X5Job()
        {
            log = new LogHelper();
            services = IOC.Resolve<IOpen5Code>();
            email = IOC.Resolve<IEmail>();
        }

        /// <summary>
        /// 作业执行入口
        /// </summary>
        /// <param name="context">作业执行上下文</param>
        public void Execute(IJobExecutionContext context)
       {
            Config = CommonHelper.GetConfigFromDataMap(context.JobDetail.JobDataMap);
            //预设节假日不开奖
            if (Config.SkipDate.Contains(CommonHelper.SCCSysDateTime.ToString("yyyyMMdd"))) return;
            LatestQiHao = context.JobDetail.JobDataMap.GetString("LatestQiHao");
            try
            {
                //服务启动时配置初始数据
                if (string.IsNullOrEmpty(LatestQiHao))
                {
                    var lastItem = services.GetLastItem(currentLottery);
                    if (lastItem != null)
                    {
                        LatestQiHao = lastItem.Term.ToString();
                    }
                }
                //第一次启动服务或最新期号为昨天的开奖期号，则自检昨天开奖数据是否抓取完毕(否则插入邮件数据)，并重置当天期号和失败列表
                if (string.IsNullOrEmpty(LatestQiHao) || !LatestQiHao.StartsWith(CommonHelper.SCCSysDateTime.ToString("yyMMdd")))
                {
                    CheckingYesterdayTheLotteryData();
                    LatestQiHao = CommonHelper.GenerateTodayQiHaoYYMMDDQQ(0);
                }
                //当最新期号不符合当天总期数，执行当天作业
                if (Convert.ToInt32(LatestQiHao.Substring(6)) != Config.TimesPerDay)
                {
                    DoTodayJobByMainUrl();
                    //DoTodayJobByBackUrl();
                }
            }
            catch (Exception ex)
            {
                log.Error(typeof(AH11X5Job), string.Format("【{0}】抓取时发生错误，错误信息【{1}】", Config.Area + Config.LotteryName, ex.Message));
            }
            //保存最新期号和失败期号列表
            context.JobDetail.JobDataMap["LatestQiHao"] = LatestQiHao;
        }

        /// <summary>
        /// 自检昨天开奖数据
        /// </summary>
        private void CheckingYesterdayTheLotteryData()
        {
            if (Config.SkipDate.Contains(CommonHelper.SCCSysDateTime.AddDays(-1).ToString("yyyyMMdd"))) return;//如果昨日设定不开奖则不自检昨日开奖数据
            //从数据库中获取昨天数据抓取失败列表
            FailedQiHaoList = services.GetYesterdayFailQQList(currentLottery, Config.TimesPerDay);
            if (FailedQiHaoList.Count > 0)
            {
                DoYesterdayFailedListByMainUrl_New();
                //DoYesterdayFailedListByBackUrl();
                foreach (var fQiHao in FailedQiHaoList)
                {
                    //将抓取失败数据推送至邮件列表，待邮件服务发送至配置管理员的邮箱中
                    if (email.AddEmail(Config.Area + Config.LotteryName, fQiHao, CommonHelper.GenerateYesterdayOpenTime(Config, fQiHao)))
                        log.Error(typeof(AH11X5Job), CommonHelper.GetJobLogError(Config, fQiHao));
                }
            }
            else if (FailedQiHaoList.Count == 0)
            {
                DoYesterdayFailedListByMainUrl();
                //DoYesterdayFailedListByBackUrl();
                foreach (var fQiHao in FailedQiHaoList)
                {
                    //将抓取失败数据推送至邮件列表，待邮件服务发送至配置管理员的邮箱中
                    if (email.AddEmail(Config.Area + Config.LotteryName, fQiHao, CommonHelper.GenerateYesterdayOpenTime(Config, fQiHao)))
                        log.Error(typeof(AH11X5Job), CommonHelper.GetJobLogError(Config, fQiHao));
                }
            }
        }

        /// <summary>
        /// 通过主站点抓取开奖数据
        /// (爱彩乐)
        /// </summary>
        private void DoTodayJobByMainUrl()
        {
            if (!string.IsNullOrEmpty(Config.MainUrl))
            {
                var OpenList = GetTodayOpenListFromMainUrl();
                if (OpenList.Count == 0) return;//无抓取数据
                var newestQiHao = OpenList.LastOrDefault().Key;
                var startQiNum = Convert.ToInt32(LatestQiHao.Substring(6)) + 1;
                var newestQiNum = Convert.ToInt32(newestQiHao.Substring(6));
                if (startQiNum > newestQiNum) return;//无最新数据
                //处理最新开奖数据
                string getQiHao = string.Empty;
                for (var i = startQiNum; i <= newestQiNum; i++)
                {
                    getQiHao = CommonHelper.GenerateTodayQiHaoYYMMDDQQ(i);
                    var matchItem = OpenList.Where(R => R.Key == getQiHao).FirstOrDefault();
                    if (matchItem.Key != null && SaveRecord(getQiHao, matchItem.Value, false))
                    {
                        //处理成功写入日志
                        log.Info(typeof(AH11X5Job), CommonHelper.GetJobMainLogInfo(Config, getQiHao));
                        LatestQiHao = getQiHao;
                    }
                }
            }
        }

        /// <summary>
        /// 通过主站抓取错误期号列表中每一个期号
        /// (爱彩乐)
        /// </summary>
        private void DoYesterdayFailedListByMainUrl()
         {
            if (!string.IsNullOrEmpty(Config.MainUrl) && FailedQiHaoList.Count > 0)
            {
                var OpenList = GetYesterdayOpenListFromMainUrl();
                if (OpenList.Count == 0) return;//无抓取数据
                var SuccessList = new List<string>();
                foreach (string failedQiHao in FailedQiHaoList)
                {
                    var matchItem = OpenList.Where(R => R.Key == failedQiHao).FirstOrDefault();
                    if (matchItem.Key != null && SaveRecord(failedQiHao, matchItem.Value, true))
                    {
                        //处理成功写入日志
                        log.Info(typeof(AH11X5Job), CommonHelper.GetJobMainLogInfo(Config, failedQiHao));
                        SuccessList.Add(failedQiHao);
                        continue;
                    }
                }
                foreach (var successQiHao in SuccessList)
                {
                    FailedQiHaoList.Remove(successQiHao);
                }
            }
        }
        /// <summary>
        /// 通过主站抓取错误期号列表中每一个期号
        /// (爱彩乐)
        /// </summary>
        private void DoYesterdayFailedListByMainUrl_New()
        {
            if (!string.IsNullOrEmpty(Config.BackUrl) && FailedQiHaoList.Count > 0)
            {
                var html = new HttpHelper().PostJson(new Dictionary<string, string>()
                    {
                        { "SiteId" , Config.SiteId },
                        { "Date",DateTime.Now.AddDays(-1).ToString("yyyy-MM-dd 00:00:00")},
                    }, Config.BackUrl);
                string HtmlResource = html;

                JObject obj = JsonConvert.DeserializeObject<JObject>(html);
                var s = obj["Data"];
                JObject obj1 = JsonConvert.DeserializeObject<JObject>(s.ToString());
                var s1 = obj1["LotteryResults"];
                var OpenList = JsonConvert.DeserializeObject<List<LotteryResults>>(s1.ToString());

                if (OpenList.Count == 0) return;//无抓取数据
                var SuccessList = new List<string>();
                foreach (string failedQiHao in FailedQiHaoList)
                {
                    var matchItem = OpenList.Where(R => R.PeriodNo.Replace("-","") == failedQiHao).FirstOrDefault();
                    string value = "";
                    foreach (var item in matchItem.LotteryNumbers)
                    {
                        value += item.Number+",";
                    }
                    if (matchItem.PeriodNo != null && SaveRecord(failedQiHao, value.Trim(','), true))
                    {
                        //处理成功写入日志
                        log.Info(typeof(AH11X5Job), CommonHelper.GetJobMainLogInfo(Config, failedQiHao));
                        SuccessList.Add(failedQiHao);
                        continue;
                    }
                }
                foreach (var successQiHao in SuccessList)
                {
                    FailedQiHaoList.Remove(successQiHao);
                }
            }
        }
        

        /// <summary>
        /// 通过主站点抓取今日最新开奖列表
        /// 爱彩乐也可以通过下面获取昨日开奖的方法获取今日开奖数据
        /// </summary>
        /// <returns></returns>
        private Dictionary<string, string> GetTodayOpenListFromMainUrl()
        {
            Dictionary<string, string> result = new Dictionary<string, string>();
            try
            {
                var HtmlResource = NetHelper.GetUrlResponse(Config.MainUrl);
                if (!string.IsNullOrWhiteSpace(HtmlResource))
                {
                    HtmlDocument doc = new HtmlDocument();
                    doc.LoadHtml(HtmlResource);
                    var table = doc.DocumentNode.SelectNodes("//*[@id='firstTbody']/tr");
                    if (table == null) return result;
                    var s = table.ToList();
                    var trs = table.ToList();
                    List<HtmlNode> tds = null, ems = null;
                    var matchQiHao = string.Empty;
                    var matchKJHaoMa = string.Empty;
                    for (var i = 0; i < trs.Count; i++)//第一行为表头
                    {
                        tds = trs[i].ChildNodes.Where(R => R.Name.ToLower() == "td").ToList();
                        if (tds.Count < 3) continue;
                        matchQiHao = tds[0].InnerText.Trim().Replace("-","");
                        ems = tds[1].ChildNodes.Where(R => R.Name.ToLower() == "em").ToList();
                        if (ems.Count < 5) continue;
                        matchKJHaoMa = string.Format("{0},{1},{2},{3},{4}", ems[0].InnerText.Trim(), ems[1].InnerText.Trim(), ems[2].InnerText.Trim(), ems[3].InnerText.Trim(), ems[4].InnerText.Trim());
                        if (!result.ContainsKey(matchQiHao))
                            result.Add(matchQiHao, matchKJHaoMa);
                    }

                    table = doc.DocumentNode.SelectNodes("//*[@id='secondTbody']/tr");
                    if (table == null) return result;
                    trs = table.ToList();
                    for (var i = 0; i < trs.Count; i++)//第一行为表头
                    {
                        tds = trs[i].ChildNodes.Where(R => R.Name.ToLower() == "td").ToList();
                        if (tds.Count < 3) continue;
                        matchQiHao = tds[0].InnerText.Trim().Replace("-", "");
                        ems = tds[1].ChildNodes.Where(R => R.Name.ToLower() == "em").ToList();
                        if (ems.Count < 5) continue;
                        matchKJHaoMa = string.Format("{0},{1},{2},{3},{4}", ems[0].InnerText.Trim(), ems[1].InnerText.Trim(), ems[2].InnerText.Trim(), ems[3].InnerText.Trim(), ems[4].InnerText.Trim());
                        if (!result.ContainsKey(matchQiHao))
                            result.Add(matchQiHao, matchKJHaoMa);
                    }
                    table = doc.DocumentNode.SelectNodes("//*[@id='thirdTbody']/tr");
                    if (table == null) return result;
                    trs = table.ToList();
                    for (var i = 0; i < trs.Count; i++)//第一行为表头
                    {
                        tds = trs[i].ChildNodes.Where(R => R.Name.ToLower() == "td").ToList();
                        if (tds.Count < 3) continue;
                        matchQiHao = tds[0].InnerText.Trim().Replace("-", "");
                        ems = tds[1].ChildNodes.Where(R => R.Name.ToLower() == "em").ToList();
                        if (ems.Count < 5) continue;
                        matchKJHaoMa = string.Format("{0},{1},{2},{3},{4}", ems[0].InnerText.Trim(), ems[1].InnerText.Trim(), ems[2].InnerText.Trim(), ems[3].InnerText.Trim(), ems[4].InnerText.Trim());
                        if (!result.ContainsKey(matchQiHao))
                            result.Add(matchQiHao, matchKJHaoMa);
                    }
                }
            }
            catch (Exception ex)
            {
                log.Error(typeof(AH11X5Job), string.Format("【{0}】通过主站点抓取今日最新开奖列表时发生错误，错误信息【{1}】", Config.Area + Config.LotteryName, ex.Message));
            }
            return result;
        }
        /// <summary>
        /// 通过主站点抓取昨日开奖列表
        /// </summary>
        /// <returns></returns>
        private Dictionary<string, string> GetYesterdayOpenListFromMainUrl()
        {
            Dictionary<string, string> result = new Dictionary<string, string>();
            try
            {
                var HtmlResource = NetHelper.GetUrlResponse(Config.MainUrl + "?action=chart&date=yesterday&id=514&async=true");
                if (!string.IsNullOrWhiteSpace(HtmlResource))
                {
                    HtmlDocument doc = new HtmlDocument();
                    doc.LoadHtml(HtmlResource);
                    var table = doc.DocumentNode.SelectNodes("//*[@id='firstTbody']/tr");
                    if (table == null) return result;
                    var s = table.ToList();
                    var trs = table.ToList();
                    List<HtmlNode> tds = null, ems = null;
                    var matchQiHao = string.Empty;
                    var matchKJHaoMa = string.Empty;
                    for (var i = 0; i < trs.Count; i++)//第一行为表头
                    {
                        tds = trs[i].ChildNodes.Where(R => R.Name.ToLower() == "td").ToList();
                        if (tds.Count < 3) continue;
                        matchQiHao = tds[0].InnerText.Trim().Replace("-", "");
                        ems = tds[1].ChildNodes.Where(R => R.Name.ToLower() == "em").ToList();
                        if (ems.Count < 5) continue;
                        matchKJHaoMa = string.Format("{0},{1},{2},{3},{4}", ems[0].InnerText.Trim(), ems[1].InnerText.Trim(), ems[2].InnerText.Trim(), ems[3].InnerText.Trim(), ems[4].InnerText.Trim());
                        if (!result.ContainsKey(matchQiHao))
                            result.Add(matchQiHao, matchKJHaoMa);
                    }

                    table = doc.DocumentNode.SelectNodes("//*[@id='secondTbody']/tr");
                    if (table == null) return result;
                    trs = table.ToList();
                    for (var i = 0; i < trs.Count; i++)//第一行为表头
                    {
                        tds = trs[i].ChildNodes.Where(R => R.Name.ToLower() == "td").ToList();
                        if (tds.Count < 3) continue;
                        matchQiHao = tds[0].InnerText.Trim().Replace("-", "");
                        ems = tds[1].ChildNodes.Where(R => R.Name.ToLower() == "em").ToList();
                        if (ems.Count < 5) continue;
                        matchKJHaoMa = string.Format("{0},{1},{2},{3},{4}", ems[0].InnerText.Trim(), ems[1].InnerText.Trim(), ems[2].InnerText.Trim(), ems[3].InnerText.Trim(), ems[4].InnerText.Trim());
                        if (!result.ContainsKey(matchQiHao))
                            result.Add(matchQiHao, matchKJHaoMa);
                    }
                    table = doc.DocumentNode.SelectNodes("//*[@id='thirdTbody']/tr");
                    if (table == null) return result;
                    trs = table.ToList();
                    for (var i = 0; i < trs.Count; i++)//第一行为表头
                    {
                        tds = trs[i].ChildNodes.Where(R => R.Name.ToLower() == "td").ToList();
                        if (tds.Count < 3) continue;
                        matchQiHao = tds[0].InnerText.Trim().Replace("-", "");
                        ems = tds[1].ChildNodes.Where(R => R.Name.ToLower() == "em").ToList();
                        if (ems.Count < 5) continue;
                        matchKJHaoMa = string.Format("{0},{1},{2},{3},{4}", ems[0].InnerText.Trim(), ems[1].InnerText.Trim(), ems[2].InnerText.Trim(), ems[3].InnerText.Trim(), ems[4].InnerText.Trim());
                        if (!result.ContainsKey(matchQiHao))
                            result.Add(matchQiHao, matchKJHaoMa);
                    }
                }
            }
            catch (Exception ex)
            {
                log.Error(typeof(AH11X5Job), string.Format("【{0}】通过主站点抓取昨日开奖列表时发生错误，错误信息【{1}】", Config.Area + Config.LotteryName, ex.Message));
            }
            return result;
        }

        /// <summary>
        /// 通过备用站点抓取开奖数据
        /// (新浪爱彩)
        /// </summary>
        private void DoTodayJobByBackUrl()
        {
            if (!string.IsNullOrEmpty(Config.BackUrl))
            {
                var OpenList = GetTodayOpenListFromBackUrl();
                if (OpenList.Count == 0) return;//无抓取数据
                var newestQiHao = OpenList.First().Key;
                var startQiNum = Convert.ToInt32(LatestQiHao.Substring(6)) + 1;
                var newestQiNum = Convert.ToInt32(newestQiHao.Substring(6));
                if (startQiNum > newestQiNum) return;//无最新数据
                //处理最新开奖数据
                var getQiHao = string.Empty;
                for (var i = startQiNum; i <= newestQiNum; i++)
                {
                    getQiHao = CommonHelper.GenerateTodayQiHaoYYMMDDQQ(i);
                    var matchItem = OpenList.Where(R => R.Key == getQiHao).FirstOrDefault();
                    if (matchItem.Key != null && SaveRecord(getQiHao, matchItem.Value, false))
                    {
                        //处理成功写入日志
                        log.Info(typeof(AH11X5Job), CommonHelper.GetJobBackLogInfo(Config, getQiHao));
                        LatestQiHao = getQiHao;
                    }
                }
            }
        }

        /// <summary>
        /// 通过备用地址抓取错误期号列表中每一个期号
        /// (新浪爱彩)
        /// </summary>
        private void DoYesterdayFailedListByBackUrl()
        {
            if (!string.IsNullOrEmpty(Config.BackUrl) && FailedQiHaoList.Count > 0)
            {
                var OpenList = GetYesterdayOpenListFromBackUrl();
                if (OpenList.Count == 0) return;//无抓取数据
                var SuccessList = new List<string>();
                foreach (var failedQiHao in FailedQiHaoList)
                {
                    var matchItem = OpenList.Where(R => R.Key == failedQiHao).FirstOrDefault();
                    if (matchItem.Key != null && SaveRecord(failedQiHao, matchItem.Value, true))
                    {
                        //处理成功写入日志
                        log.Info(typeof(AH11X5Job), CommonHelper.GetJobBackLogInfo(Config, failedQiHao));
                        SuccessList.Add(failedQiHao);
                        continue;
                    }
                }
                foreach (var successQiHao in SuccessList)
                {
                    FailedQiHaoList.Remove(successQiHao);
                }
            }
        }
        /// <summary>
        /// 通过备用站点抓取今日最新开奖数据
        /// </summary>
        /// <returns></returns>
        private Dictionary<string, string> GetTodayOpenListFromBackUrl()
        {
            var postData = string.Format("gameIndex=312&searchDate={0}&gameFrom=", CommonHelper.SCCSysDateTime.ToString("yyyy-MM-dd"));
            return GetOpenListFromBackUrl(postData);
        }
        /// <summary>
        /// 通过备用站点抓取昨日开奖数据
        /// </summary>
        /// <returns></returns>
        private Dictionary<string, string> GetYesterdayOpenListFromBackUrl()
        {
            var postData = string.Format("gameIndex=312&searchDate={0}&gameFrom=", CommonHelper.SCCSysDateTime.AddDays(-1).ToString("yyyy-MM-dd"));
            return GetOpenListFromBackUrl(postData);
        }
        /// <summary>
        /// 通过备用站点抓取开奖数据
        /// </summary>
        /// <param name="PostData"></param>
        /// <returns></returns>
        private Dictionary<string, string> GetOpenListFromBackUrl(string PostData)
        {
            Dictionary<string, string> result = new Dictionary<string, string>();
            try
            {
                var HtmlResource = NetHelper.GetUrlResponse(Config.BackUrl, "POST", PostData);
                if (!string.IsNullOrWhiteSpace(HtmlResource))
                {
                    //JObject obj = JsonConvert.DeserializeObject<JObject>(HtmlResource);
                    //if (obj == null || obj["resultHtml"] == null) return result;
                    //var trString = obj["resultHtml"].ToString();
                    //if (string.IsNullOrWhiteSpace(trString)) return result;
                    //var tableString = string.Format("<table>{0}</table>", trString.Replace("<tr>", "</tr><tr>").Replace("<tr class='bg'>", "</tr><tr>").Substring(5) + "</tr>");
                    //if (string.IsNullOrWhiteSpace(tableString)) return result;
                    HtmlDocument doc = new HtmlDocument();
                    doc.LoadHtml(HtmlResource);
                    var table = doc.DocumentNode.SelectNodes("//*[@id='firstTbody']/tr");
                    if (table == null) return result;
                    var s = table.ToList();
                    var trs = table.ToList();
                    List<HtmlNode> tds = null, ems = null;
                    var matchQiHao = string.Empty;
                    var matchKJHaoMa = string.Empty;
                    for (var i = 0; i < trs.Count; i++)//第一行为表头
                    {
                        tds = trs[i].ChildNodes.Where(R => R.Name.ToLower() == "td").ToList();
                        if (tds.Count < 3) continue;
                        matchQiHao = tds[0].InnerText.Trim().Replace("-", "");
                        ems = tds[1].ChildNodes.Where(R => R.Name.ToLower() == "em").ToList();
                        if (ems.Count < 5) continue;
                        matchKJHaoMa = string.Format("{0},{1},{2},{3},{4}", ems[0].InnerText.Trim(), ems[1].InnerText.Trim(), ems[2].InnerText.Trim(), ems[3].InnerText.Trim(), ems[4].InnerText.Trim());
                        if (!result.ContainsKey(matchQiHao))
                            result.Add(matchQiHao, matchKJHaoMa);
                    }
                    table = doc.DocumentNode.SelectNodes("//*[@id='secondTbody']/tr");
                    if (table == null) return result;
                    trs = table.ToList();
                    for (var i = 0; i < trs.Count; i++)//第一行为表头
                    {
                        tds = trs[i].ChildNodes.Where(R => R.Name.ToLower() == "td").ToList();
                        if (tds.Count < 3) continue;
                        matchQiHao = tds[0].InnerText.Trim().Replace("-", "");
                        ems = tds[1].ChildNodes.Where(R => R.Name.ToLower() == "em").ToList();
                        if (ems.Count < 5) continue;
                        matchKJHaoMa = string.Format("{0},{1},{2},{3},{4}", ems[0].InnerText.Trim(), ems[1].InnerText.Trim(), ems[2].InnerText.Trim(), ems[3].InnerText.Trim(), ems[4].InnerText.Trim());
                        if (!result.ContainsKey(matchQiHao))
                            result.Add(matchQiHao, matchKJHaoMa);
                    }
                    table = doc.DocumentNode.SelectNodes("//*[@id='thirdTbody']/tr");
                    if (table == null) return result;
                    trs = table.ToList();
                    for (var i = 0; i < trs.Count; i++)//第一行为表头
                    {
                        tds = trs[i].ChildNodes.Where(R => R.Name.ToLower() == "td").ToList();
                        if (tds.Count < 3) continue;
                        matchQiHao = tds[0].InnerText.Trim().Replace("-", "");
                        ems = tds[1].ChildNodes.Where(R => R.Name.ToLower() == "em").ToList();
                        if (ems.Count < 5) continue;
                        matchKJHaoMa = string.Format("{0},{1},{2},{3},{4}", ems[0].InnerText.Trim(), ems[1].InnerText.Trim(), ems[2].InnerText.Trim(), ems[3].InnerText.Trim(), ems[4].InnerText.Trim());
                        if (!result.ContainsKey(matchQiHao))
                            result.Add(matchQiHao, matchKJHaoMa);
                    }
                    //var table = doc.DocumentNode.SelectSingleNode("//table");
                    //if (table == null) return result;
                    //var trs = table.ChildNodes.Where(node => node.Name == "tr").ToList();
                    //string matchQiHao = string.Empty;
                    //string matchKJHaoMa = string.Empty;
                    //for (var i = 0; i < trs.Count; i++)
                    //{
                    //    var tds = trs[i].ChildNodes.Where(node => node.Name == "td").ToList();
                    //    if (tds.Count < 3) continue;
                    //    matchQiHao = tds[0].InnerText.Replace("-", string.Empty).Replace("期", string.Empty).Trim().Substring(2);
                    //    matchKJHaoMa = tds[2].InnerText.Trim();
                    //    if (!result.ContainsKey(matchQiHao))
                    //        result.Add(matchQiHao, matchKJHaoMa);
                    //}
                }
            }
            catch (Exception ex)
            {
                log.Error(typeof(AH11X5Job), string.Format("【{0}】通过备用站点抓取开奖列表时发生错误，错误信息【{1}】", Config.Area + Config.LotteryName, ex.Message));
            }
            return result;
        }

        /// <summary>
        /// 将此彩种指定期号和开奖号码保存至数据库
        /// </summary>
        /// <param name="QiHao">期号</param>
        /// <param name="OpenCode">开奖号码(形如01,02,03,04,05)</param>
        /// <param name="IsYesterdayRecord">是否是保存昨天的记录</param>
        /// <returns></returns>
        private bool SaveRecord(string QiHao, string OpenCode, bool IsYesterdayRecord)
        {
            if (!string.IsNullOrWhiteSpace(QiHao) && !string.IsNullOrWhiteSpace(OpenCode))
            {
                OpenCode5Model model = new OpenCode5Model();
                model.Term = Convert.ToInt64(QiHao);
                var haoMaArray = OpenCode.Split(new char[] { ',' });
                model.OpenCode1 = Convert.ToInt32(haoMaArray[0]);
                model.OpenCode2 = Convert.ToInt32(haoMaArray[1]);
                model.OpenCode3 = Convert.ToInt32(haoMaArray[2]);
                model.OpenCode4 = Convert.ToInt32(haoMaArray[3]);
                model.OpenCode5 = Convert.ToInt32(haoMaArray[4]);
                if (IsYesterdayRecord)
                    model.OpenTime = CommonHelper.GenerateYesterdayOpenTime(Config, QiHao);
                else
                    model.OpenTime = CommonHelper.GenerateTodayOpenTime(Config, QiHao);
                return services.AddOpen5Code(currentLottery, model);
            }
            return false;
        }

        #region Attribute
        /// <summary>
        /// 配置信息
        /// </summary>
        private SCCConfig Config = null;
        /// <summary>
        /// 当天抓取的最新一期期号
        /// </summary>
        private string LatestQiHao = null;
        /// <summary>
        /// 当天抓取失败列表
        /// </summary>
        private List<string> FailedQiHaoList = null;
        /// <summary>
        /// 日志对象
        /// </summary>
        private LogHelper log = null;
        /// <summary>
        /// 数据服务
        /// </summary>
        private IOpen5Code services = null;
        /// <summary>
        /// 当前彩种
        /// </summary>
        private SCCLottery currentLottery
        {
            get
            {
                return SCCLottery.AnHui11x5;
            }
        }
        /// <summary>
        /// 邮件接口
        /// </summary>
        private IEmail email = null;
        #endregion
    }
}
