﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;

using Quartz;
using SCC.Common;
using SCC.Models;
using SCC.Interface;
using HtmlAgilityPack;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;

namespace SCC.Crawler.GP
{
    /// <summary>
    /// 数据爬取类
    /// 云南11选5
    /// </summary>
    [DisallowConcurrentExecution]
    [PersistJobDataAfterExecution]
    public class YN11X5Job : IJob
    {
        /// <summary>
        /// 构造函数
        /// </summary>
        public YN11X5Job()
        {
            log = new LogHelper();
            services = IOC.Resolve<IOpen5Code>();
            email = IOC.Resolve<IEmail>();
        }

        /// <summary>
        /// 作业执行入口
        /// </summary>
        /// <param name="context">作业执行上下文</param>
        public void Execute(IJobExecutionContext context)
        {
            Config = CommonHelper.GetConfigFromDataMap(context.JobDetail.JobDataMap);
            //预设节假日不开奖
            if (Config.SkipDate.Contains(CommonHelper.SCCSysDateTime.ToString("yyyyMMdd"))) return;
            LatestQiHao = context.JobDetail.JobDataMap.GetString("LatestQiHao");
            try
            {
                //服务启动时配置初始数据
                if (string.IsNullOrEmpty(LatestQiHao))
                {
                    var lastItem = services.GetLastItem(currentLottery);
                    if (lastItem != null)
                    {
                        LatestQiHao = lastItem.Term.ToString();
                    }
                }
                //第一次启动服务或最新期号为昨天的开奖期号，则自检昨天开奖数据是否抓取完毕(否则插入邮件数据)，并重置当天期号和失败列表
                if (string.IsNullOrEmpty(LatestQiHao) || !LatestQiHao.StartsWith(CommonHelper.SCCSysDateTime.ToString("yyMMdd")))
                {
                    CheckingYesterdayTheLotteryData();
                    LatestQiHao = CommonHelper.GenerateTodayQiHaoYYMMDDQQ(0);
                }
                //当最新期号不符合当天总期数，执行当天作业
                if (Convert.ToInt32(LatestQiHao.Substring(6)) != Config.TimesPerDay)
                {
                    DoTodayJobByDB();
                    //DoTodayJobByMainUrl();
                    //DoTodayJobByBackUrl();
                }
            }
            catch (Exception ex)
            {
                log.Error(typeof(YN11X5Job), string.Format("【{0}】抓取时发生错误，错误信息【{1}】", Config.Area + Config.LotteryName, ex.Message));
            }
            //保存最新期号和失败期号列表
            context.JobDetail.JobDataMap["LatestQiHao"] = LatestQiHao;
        }

        /// <summary>
        /// 自检昨天开奖数据
        /// </summary>
        private void CheckingYesterdayTheLotteryData()
        {
            if (Config.SkipDate.Contains(CommonHelper.SCCSysDateTime.AddDays(-1).ToString("yyyyMMdd"))) return;//如果昨日设定不开奖则不自检昨日开奖数据
            //从数据库中获取昨天数据抓取失败列表
            FailedQiHaoList = services.GetYesterdayFailQQList(currentLottery, Config.TimesPerDay);
            if (FailedQiHaoList.Count > 0)
            {
                DoYesterdayFailedListByMainUrl_New();
               // DoYesterdayFailedListByMainUrl();
                //DoYesterdayFailedListByBackUrl();
                foreach (var fQiHao in FailedQiHaoList)
                {
                    //将抓取失败数据推送至邮件列表，待邮件服务发送至配置管理员的邮箱中
                    if (email.AddEmail(Config.Area + Config.LotteryName, fQiHao, CommonHelper.GenerateYesterdayOpenTime(Config, fQiHao)))
                        log.Error(typeof(YN11X5Job), CommonHelper.GetJobLogError(Config, fQiHao));
                }
            }
        }
        /// <summary>
        /// 新开奖号数据库拿取
        /// </summary>
        private void DoYesterdayFailedListByMainUrl_New()
        {
            var OpenList = GetYesterdayOpenListFromDB_new();
            if (OpenList.Count == 0) return;//无抓取数据
            var SuccessList = new List<string>();
            foreach (string failedQiHao in FailedQiHaoList)
            {
                var matchItem = OpenList.Where(R => R.Key == failedQiHao).FirstOrDefault();
                if (matchItem.Key != null && SaveRecord(failedQiHao, matchItem.Value, true))
                {
                    //处理成功写入日志
                    log.Info(typeof(HeBei11X5Job), CommonHelper.GetJobMainLogInfo(Config, failedQiHao));
                    SuccessList.Add(failedQiHao);
                    continue;
                }
            }
            foreach (var successQiHao in SuccessList)
            {
                FailedQiHaoList.Remove(successQiHao);
            }
        }
        /// <summary>
        /// 通过数据库点抓取昨日开奖列表
        /// </summary>
        /// <returns></returns>
        private Dictionary<string, string> GetYesterdayOpenListFromDB_new()
        {
            Dictionary<string, string> result = new Dictionary<string, string>();
            try
            {
                var list = services.GetYesterdayFailQQListDB5(currentLottery, Config.TimesPerDay);
                result = list;
            }
            catch (Exception ex)
            {
                log.Error(typeof(HeBei11X5Job), string.Format("【{0}】通过主站点抓取昨日开奖列表时发生错误，错误信息【{1}】", Config.Area + Config.LotteryName, ex.Message));
            }
            return result;
        }
        /// <summary>
        /// 通过数据库获取开奖数据
        /// </summary>
        private void DoTodayJobByDB()
        {
            
                var OpenList = services.GetYesterdayFailQQListDB5_toDay(currentLottery, Config.TimesPerDay);
                if (OpenList.Count == 0) return;//无抓取数据
                var newestQiHao = OpenList.OrderBy(a=>a.Key).Last().Key;
                var startQiNum = Convert.ToInt32(LatestQiHao.Substring(6)) + 1;
                var newestQiNum = Convert.ToInt32(newestQiHao.Substring(6));
                if (startQiNum > newestQiNum) return;//无最新数据
                //处理最新开奖数据
                string getQiHao = string.Empty;
                for (var i = startQiNum; i <= newestQiNum; i++)
                {
                    getQiHao = CommonHelper.GenerateTodayQiHaoYYMMDDQQ(i);
                    var matchItem = OpenList.Where(R => R.Key == getQiHao).FirstOrDefault();
                    if (matchItem.Key != null && SaveRecord(getQiHao, matchItem.Value, false))
                    {
                        //处理成功写入日志
                        log.Info(typeof(ZJ11X5Job), CommonHelper.GetJobMainLogInfo(Config, getQiHao));
                        LatestQiHao = getQiHao;
                    }
                }
        }
        /// <summary>
        /// 通过主站点抓取开奖数据
        /// (云南体彩网)
        /// </summary>
        private void DoTodayJobByMainUrl()
        {
            if (!string.IsNullOrEmpty(Config.MainUrl))
            {
                var OpenList = GetTodayOpenListFromMainUrl();
                if (OpenList.Count == 0) return;//无抓取数据
                var newestQiHao = OpenList.Last().Key;
                var startQiNum = Convert.ToInt32(LatestQiHao.Substring(6)) + 1;
                var newestQiNum = Convert.ToInt32(newestQiHao.Substring(6));
                if (startQiNum > newestQiNum) return;//无最新数据
                //处理最新开奖数据
                string getQiHao = string.Empty;
                for (var i = startQiNum; i <= newestQiNum; i++)
                {
                    getQiHao = CommonHelper.GenerateTodayQiHaoYYMMDDQQ(i);
                    var matchItem = OpenList.Where(R => R.Key == getQiHao).FirstOrDefault();
                    if (matchItem.Key != null && SaveRecord(getQiHao, matchItem.Value, false))
                    {
                        //处理成功写入日志
                        log.Info(typeof(YN11X5Job), CommonHelper.GetJobMainLogInfo(Config, getQiHao));
                        LatestQiHao = getQiHao;
                    }
                }
            }
        }

        /// <summary>
        /// 通过主站抓取错误期号列表中每一个期号
        /// (云南体彩网)
        /// </summary>
        private void DoYesterdayFailedListByMainUrl()
        {
            if (!string.IsNullOrEmpty(Config.MainUrl) && FailedQiHaoList.Count > 0)
            {
                var OpenList = GetYesterdayOpenListFromMainUrl();
                if (OpenList.Count == 0) return;//无抓取数据
                var SuccessList = new List<string>();
                foreach (string failedQiHao in FailedQiHaoList)
                {
                    var matchItem = OpenList.Where(R => R.Key == failedQiHao).FirstOrDefault();
                    if (matchItem.Key != null && SaveRecord(failedQiHao, matchItem.Value, true))
                    {
                        //处理成功写入日志
                        log.Info(typeof(YN11X5Job), CommonHelper.GetJobMainLogInfo(Config, failedQiHao));
                        SuccessList.Add(failedQiHao);
                        continue;
                    }
                }
                foreach (var successQiHao in SuccessList)
                {
                    FailedQiHaoList.Remove(successQiHao);
                }
            }
        }

        /// <summary>
        /// 通过主站点抓取今日最新开奖列表
        /// 爱彩乐也可以通过下面获取昨日开奖的方法获取今日开奖数据
        /// </summary>
        /// <returns></returns>
        private Dictionary<string, string> GetTodayOpenListFromMainUrl()
        {
            var todaypostdata = "__EVENTTARGET=drop1&__EVENTARGUMENT=&__LASTFOCUS=&__VIEWSTATE=%2FwEPDwULLTEwNzQzMzg2NzYPZBYCAgMPZBYEAgEPEGRkFgFmZAIDDxYCHgRUZXh0BbTSATx0YWJsZSB3aWR0aD0iMTI1MiIgY2xhc3M9InRiayI%2BDQo8dHI%2BPHRoIHdpZHRoPSI3NSIgcm93c3Bhbj0iMiI%2B5pel5pyfPC90aD48dGggd2lkdGg9IjcwIiByb3dzcGFuPSIyIj7mnJ%2Flj7c8L3RoPjx0aCB3aWR0aD0iMTAwIiByb3dzcGFuPSIyIiBjbGFzcz0iZGIxIj7lvIDlpZblj7fnoIE8L3RoPjx0aCB3aWR0aD0iMTAwIiByb3dzcGFuPSIyIiBjbGFzcz0iZGIxIj7nu4TpgIk8L3RoPjx0aCBjb2xzcGFuPSIxMSIgY2xhc3M9ImRiMSI%2B5byA5aWW5Y%2B356CB5YiG5biDPC90aD48dGggY29sc3Bhbj0iNiIgY2xhc3M9ImRiMSI%2B5aWH5pWw5Liq5pWw6LWw5Yq%2FPC90aD48dGggY29sc3Bhbj0iNiIgY2xhc3M9ImRiMSI%2B5aSn5pWw5Liq5pWw6LWw5Yq%2FPC90aD48dGggY29sc3Bhbj0iNiI%2B6LSo5pWw5Liq5pWw6LWw5Yq%2FPC90aD48L3RyPg0KICA8dHIgY2xhc3M9ImRiMiBiZ3RoIj48dGg%2BMDE8L3RoPjx0aD4wMjwvdGg%2BPHRoPjAzPC90aD48dGg%2BMDQ8L3RoPjx0aD4wNTwvdGg%2BPHRoPjA2PC90aD48dGg%2BMDc8L3RoPjx0aD4wODwvdGg%2BPHRoPjA5PC90aD48dGg%2BMTA8L3RoPjx0aCBjbGFzcz0iZGIxIj4xMTwvdGg%2BPHRoPjA8L3RoPjx0aD4xPC90aD48dGg%2BMjwvdGg%2BPHRoPjM8L3RoPjx0aD40PC90aD48dGggY2xhc3M9ImRiMSI%2BNTwvdGg%2BPHRoPjA8L3RoPjx0aD4xPC90aD48dGg%2BMjwvdGg%2BPHRoPjM8L3RoPjx0aD40PC90aD48dGggY2xhc3M9ImRiMSI%2BNTwvdGg%2BPHRoPjA8L3RoPjx0aD4xPC90aD48dGg%2BMjwvdGg%2BPHRoPjM8L3RoPjx0aD40PC90aD48dGg%2BNTwvdGg%2BPC90cj4NCjx0cj4NCjx0ZD4yMDE3LTAyLTA4PC90ZD48dGQgY2xhc3M9InQwMiI%2BMTcwMjA4NzI8L3RkPjx0ZCBjbGFzcz0iZGIxIGZ3Ij4wNiAwOCAwNCAxMSAwOTwvdGQ%2BPHRkIGNsYXNzPSJkYjEgZnciPjA0IDA2IDA4IDA5IDExPC90ZD48dGQgY2xhc3M9ImIwNSI%2BMzwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjQ8L3RkPjx0ZCBjbGFzcz0iYjA1Ij4xPC90ZD48dGQgY2xhc3M9ImIwNSBiYWxsX3JlZCI%2BNDwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjM8L3RkPjx0ZCBjbGFzcz0iYjA1IGJhbGxfcmVkIj42PC90ZD48dGQgY2xhc3M9ImIwNSI%2BMjwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQiPjg8L3RkPjx0ZCBjbGFzcz0iYjA1IGJhbGxfcmVkIj45PC90ZD48dGQgY2xhc3M9ImIwNSI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQgZGIxIj4xMTwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjIwMjk8L3RkPjx0ZCBjbGFzcz0iYjAyIj4zNDwvdGQ%2BPHRkIGNsYXNzPSJiMDIgcmVjdF9ibHVlIj4yPC90ZD48dGQgY2xhc3M9ImIwMiI%2BMjwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjM8L3RkPjx0ZCBjbGFzcz0iYjAyIGRiMSI%2BMTk8L3RkPjx0ZCBjbGFzcz0iYjAxIj4yODg8L3RkPjx0ZCBjbGFzcz0iYjAxIj4yMjwvdGQ%2BPHRkIGNsYXNzPSJiMDEiPjQ8L3RkPjx0ZCBjbGFzcz0iYjAxIj4xPC90ZD48dGQgY2xhc3M9ImIwMSByZWN0X3llbGxvdyI%2BNDwvdGQ%2BPHRkIGNsYXNzPSJiMDEgZGIxIj4yPC90ZD48dGQgY2xhc3M9ImIwMiI%2BODU8L3RkPjx0ZCBjbGFzcz0iYjAyIHJlY3RfcmVkIj4xPC90ZD48dGQgY2xhc3M9ImIwMiI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjM8L3RkPjx0ZCBjbGFzcz0iYjAyIj45PC90ZD48dGQgY2xhc3M9ImIwMiI%2BMjM8L3RkPjx0cj4NCjx0ZD4yMDE3LTAyLTA4PC90ZD48dGQgY2xhc3M9InQwMiI%2BMTcwMjA4NzM8L3RkPjx0ZCBjbGFzcz0iZGIxIGZ3Ij4xMCAwNyAwNCAwNSAwODwvdGQ%2BPHRkIGNsYXNzPSJkYjEgZnciPjA0IDA1IDA3IDA4IDEwPC90ZD48dGQgY2xhc3M9ImIwNSI%2BNDwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjU8L3RkPjx0ZCBjbGFzcz0iYjA1Ij4yPC90ZD48dGQgY2xhc3M9ImIwNSBiYWxsX3JlZCI%2BNDwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQiPjU8L3RkPjx0ZCBjbGFzcz0iYjA1Ij4xPC90ZD48dGQgY2xhc3M9ImIwNSBiYWxsX3JlZCI%2BNzwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQiPjg8L3RkPjx0ZCBjbGFzcz0iYjA1Ij4xPC90ZD48dGQgY2xhc3M9ImIwNSBiYWxsX3JlZCI%2BMTA8L3RkPjx0ZCBjbGFzcz0iYjA1IGRiMSI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjIwMzA8L3RkPjx0ZCBjbGFzcz0iYjAyIj4zNTwvdGQ%2BPHRkIGNsYXNzPSJiMDIgcmVjdF9ibHVlIj4yPC90ZD48dGQgY2xhc3M9ImIwMiI%2BMzwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjQ8L3RkPjx0ZCBjbGFzcz0iYjAyIGRiMSI%2BMjA8L3RkPjx0ZCBjbGFzcz0iYjAxIj4yODk8L3RkPjx0ZCBjbGFzcz0iYjAxIj4yMzwvdGQ%2BPHRkIGNsYXNzPSJiMDEiPjU8L3RkPjx0ZCBjbGFzcz0iYjAxIHJlY3RfeWVsbG93Ij4zPC90ZD48dGQgY2xhc3M9ImIwMSI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDEgZGIxIj4zPC90ZD48dGQgY2xhc3M9ImIwMiI%2BODY8L3RkPjx0ZCBjbGFzcz0iYjAyIj4xPC90ZD48dGQgY2xhc3M9ImIwMiByZWN0X3JlZCI%2BMjwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjQ8L3RkPjx0ZCBjbGFzcz0iYjAyIj4xMDwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjI0PC90ZD48dHI%2BDQo8dGQ%2BMjAxNy0wMi0wODwvdGQ%2BPHRkIGNsYXNzPSJ0MDIiPjE3MDIwODc0PC90ZD48dGQgY2xhc3M9ImRiMSBmdyI%2BMDEgMDIgMDUgMDYgMDQ8L3RkPjx0ZCBjbGFzcz0iZGIxIGZ3Ij4wMSAwMiAwNCAwNSAwNjwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQiPjE8L3RkPjx0ZCBjbGFzcz0iYjA1IGJhbGxfcmVkIj4yPC90ZD48dGQgY2xhc3M9ImIwNSI%2BMzwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQiPjQ8L3RkPjx0ZCBjbGFzcz0iYjA1IGJhbGxfcmVkIj41PC90ZD48dGQgY2xhc3M9ImIwNSBiYWxsX3JlZCI%2BNjwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjE8L3RkPjx0ZCBjbGFzcz0iYjA1Ij4xPC90ZD48dGQgY2xhc3M9ImIwNSI%2BMjwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjE8L3RkPjx0ZCBjbGFzcz0iYjA1IGRiMSI%2BMjwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjIwMzE8L3RkPjx0ZCBjbGFzcz0iYjAyIj4zNjwvdGQ%2BPHRkIGNsYXNzPSJiMDIgcmVjdF9ibHVlIj4yPC90ZD48dGQgY2xhc3M9ImIwMiI%2BNDwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjU8L3RkPjx0ZCBjbGFzcz0iYjAyIGRiMSI%2BMjE8L3RkPjx0ZCBjbGFzcz0iYjAxIj4yOTA8L3RkPjx0ZCBjbGFzcz0iYjAxIHJlY3RfeWVsbG93Ij4xPC90ZD48dGQgY2xhc3M9ImIwMSI%2BNjwvdGQ%2BPHRkIGNsYXNzPSJiMDEiPjE8L3RkPjx0ZCBjbGFzcz0iYjAxIj4yPC90ZD48dGQgY2xhc3M9ImIwMSBkYjEiPjQ8L3RkPjx0ZCBjbGFzcz0iYjAyIj44NzwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjI8L3RkPjx0ZCBjbGFzcz0iYjAyIj4xPC90ZD48dGQgY2xhc3M9ImIwMiByZWN0X3JlZCI%2BMzwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjExPC90ZD48dGQgY2xhc3M9ImIwMiI%2BMjU8L3RkPjx0cj4NCjx0ZD4yMDE3LTAyLTA4PC90ZD48dGQgY2xhc3M9InQwMiI%2BMTcwMjA4NzU8L3RkPjx0ZCBjbGFzcz0iZGIxIGZ3Ij4wNSAwMSAwOSAxMSAwNDwvdGQ%2BPHRkIGNsYXNzPSJkYjEgZnciPjAxIDA0IDA1IDA5IDExPC90ZD48dGQgY2xhc3M9ImIwNSBiYWxsX3JlZCI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjE8L3RkPjx0ZCBjbGFzcz0iYjA1Ij40PC90ZD48dGQgY2xhc3M9ImIwNSBiYWxsX3JlZCI%2BNDwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQiPjU8L3RkPjx0ZCBjbGFzcz0iYjA1Ij4xPC90ZD48dGQgY2xhc3M9ImIwNSI%2BMjwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjI8L3RkPjx0ZCBjbGFzcz0iYjA1IGJhbGxfcmVkIj45PC90ZD48dGQgY2xhc3M9ImIwNSI%2BMjwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQgZGIxIj4xMTwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjIwMzI8L3RkPjx0ZCBjbGFzcz0iYjAyIj4zNzwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjE8L3RkPjx0ZCBjbGFzcz0iYjAyIj41PC90ZD48dGQgY2xhc3M9ImIwMiByZWN0X2JsdWUiPjQ8L3RkPjx0ZCBjbGFzcz0iYjAyIGRiMSI%2BMjI8L3RkPjx0ZCBjbGFzcz0iYjAxIj4yOTE8L3RkPjx0ZCBjbGFzcz0iYjAxIj4xPC90ZD48dGQgY2xhc3M9ImIwMSByZWN0X3llbGxvdyI%2BMjwvdGQ%2BPHRkIGNsYXNzPSJiMDEiPjI8L3RkPjx0ZCBjbGFzcz0iYjAxIj4zPC90ZD48dGQgY2xhc3M9ImIwMSBkYjEiPjU8L3RkPjx0ZCBjbGFzcz0iYjAyIj44ODwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjM8L3RkPjx0ZCBjbGFzcz0iYjAyIj4yPC90ZD48dGQgY2xhc3M9ImIwMiByZWN0X3JlZCI%2BMzwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjEyPC90ZD48dGQgY2xhc3M9ImIwMiI%2BMjY8L3RkPjx0ciBjbGFzcz0iZGIyIj4NCjx0ZD4yMDE3LTAyLTA4PC90ZD48dGQgY2xhc3M9InQwMiI%2BMTcwMjA4NzY8L3RkPjx0ZCBjbGFzcz0iZGIxIGZ3Ij4wOSAwMyAwNCAwNyAxMTwvdGQ%2BPHRkIGNsYXNzPSJkYjEgZnciPjAzIDA0IDA3IDA5IDExPC90ZD48dGQgY2xhc3M9ImIwNSI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjI8L3RkPjx0ZCBjbGFzcz0iYjA1IGJhbGxfcmVkIj4zPC90ZD48dGQgY2xhc3M9ImIwNSBiYWxsX3JlZCI%2BNDwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjE8L3RkPjx0ZCBjbGFzcz0iYjA1Ij4yPC90ZD48dGQgY2xhc3M9ImIwNSBiYWxsX3JlZCI%2BNzwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjM8L3RkPjx0ZCBjbGFzcz0iYjA1IGJhbGxfcmVkIj45PC90ZD48dGQgY2xhc3M9ImIwNSI%2BMzwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQgZGIxIj4xMTwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjIwMzM8L3RkPjx0ZCBjbGFzcz0iYjAyIj4zODwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjI8L3RkPjx0ZCBjbGFzcz0iYjAyIj42PC90ZD48dGQgY2xhc3M9ImIwMiByZWN0X2JsdWUiPjQ8L3RkPjx0ZCBjbGFzcz0iYjAyIGRiMSI%2BMjM8L3RkPjx0ZCBjbGFzcz0iYjAxIj4yOTI8L3RkPjx0ZCBjbGFzcz0iYjAxIj4yPC90ZD48dGQgY2xhc3M9ImIwMSI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDEgcmVjdF95ZWxsb3ciPjM8L3RkPjx0ZCBjbGFzcz0iYjAxIj40PC90ZD48dGQgY2xhc3M9ImIwMSBkYjEiPjY8L3RkPjx0ZCBjbGFzcz0iYjAyIj44OTwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjQ8L3RkPjx0ZCBjbGFzcz0iYjAyIj4zPC90ZD48dGQgY2xhc3M9ImIwMiByZWN0X3JlZCI%2BMzwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjEzPC90ZD48dGQgY2xhc3M9ImIwMiI%2BMjc8L3RkPjx0cj4NCjx0ZD4yMDE3LTAyLTA4PC90ZD48dGQgY2xhc3M9InQwMiI%2BMTcwMjA4Nzc8L3RkPjx0ZCBjbGFzcz0iZGIxIGZ3Ij4wNSAwOCAwMiAxMSAwNDwvdGQ%2BPHRkIGNsYXNzPSJkYjEgZnciPjAyIDA0IDA1IDA4IDExPC90ZD48dGQgY2xhc3M9ImIwNSI%2BMjwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQiPjI8L3RkPjx0ZCBjbGFzcz0iYjA1Ij4xPC90ZD48dGQgY2xhc3M9ImIwNSBiYWxsX3JlZCI%2BNDwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQiPjU8L3RkPjx0ZCBjbGFzcz0iYjA1Ij4zPC90ZD48dGQgY2xhc3M9ImIwNSI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQiPjg8L3RkPjx0ZCBjbGFzcz0iYjA1Ij4xPC90ZD48dGQgY2xhc3M9ImIwNSI%2BNDwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQgZGIxIj4xMTwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjIwMzQ8L3RkPjx0ZCBjbGFzcz0iYjAyIj4zOTwvdGQ%2BPHRkIGNsYXNzPSJiMDIgcmVjdF9ibHVlIj4yPC90ZD48dGQgY2xhc3M9ImIwMiI%2BNzwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjE8L3RkPjx0ZCBjbGFzcz0iYjAyIGRiMSI%2BMjQ8L3RkPjx0ZCBjbGFzcz0iYjAxIj4yOTM8L3RkPjx0ZCBjbGFzcz0iYjAxIj4zPC90ZD48dGQgY2xhc3M9ImIwMSByZWN0X3llbGxvdyI%2BMjwvdGQ%2BPHRkIGNsYXNzPSJiMDEiPjE8L3RkPjx0ZCBjbGFzcz0iYjAxIj41PC90ZD48dGQgY2xhc3M9ImIwMSBkYjEiPjc8L3RkPjx0ZCBjbGFzcz0iYjAyIj45MDwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjU8L3RkPjx0ZCBjbGFzcz0iYjAyIj40PC90ZD48dGQgY2xhc3M9ImIwMiByZWN0X3JlZCI%2BMzwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjE0PC90ZD48dGQgY2xhc3M9ImIwMiI%2BMjg8L3RkPjx0cj4NCjx0ZD4yMDE3LTAyLTA4PC90ZD48dGQgY2xhc3M9InQwMiI%2BMTcwMjA4Nzg8L3RkPjx0ZCBjbGFzcz0iZGIxIGZ3Ij4wMSAwMyAwNCAwOCAwNjwvdGQ%2BPHRkIGNsYXNzPSJkYjEgZnciPjAxIDAzIDA0IDA2IDA4PC90ZD48dGQgY2xhc3M9ImIwNSBiYWxsX3JlZCI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjE8L3RkPjx0ZCBjbGFzcz0iYjA1IGJhbGxfcmVkIj4zPC90ZD48dGQgY2xhc3M9ImIwNSBiYWxsX3JlZCI%2BNDwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjE8L3RkPjx0ZCBjbGFzcz0iYjA1IGJhbGxfcmVkIj42PC90ZD48dGQgY2xhc3M9ImIwNSI%2BMjwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQiPjg8L3RkPjx0ZCBjbGFzcz0iYjA1Ij4yPC90ZD48dGQgY2xhc3M9ImIwNSI%2BNTwvdGQ%2BPHRkIGNsYXNzPSJiMDUgZGIxIj4xPC90ZD48dGQgY2xhc3M9ImIwMiI%2BMjAzNTwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjQwPC90ZD48dGQgY2xhc3M9ImIwMiByZWN0X2JsdWUiPjI8L3RkPjx0ZCBjbGFzcz0iYjAyIj44PC90ZD48dGQgY2xhc3M9ImIwMiI%2BMjwvdGQ%2BPHRkIGNsYXNzPSJiMDIgZGIxIj4yNTwvdGQ%2BPHRkIGNsYXNzPSJiMDEiPjI5NDwvdGQ%2BPHRkIGNsYXNzPSJiMDEiPjQ8L3RkPjx0ZCBjbGFzcz0iYjAxIHJlY3RfeWVsbG93Ij4yPC90ZD48dGQgY2xhc3M9ImIwMSI%2BMjwvdGQ%2BPHRkIGNsYXNzPSJiMDEiPjY8L3RkPjx0ZCBjbGFzcz0iYjAxIGRiMSI%2BODwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjkxPC90ZD48dGQgY2xhc3M9ImIwMiI%2BNjwvdGQ%2BPHRkIGNsYXNzPSJiMDIgcmVjdF9yZWQiPjI8L3RkPjx0ZCBjbGFzcz0iYjAyIj4xPC90ZD48dGQgY2xhc3M9ImIwMiI%2BMTU8L3RkPjx0ZCBjbGFzcz0iYjAyIj4yOTwvdGQ%2BPHRyPg0KPHRkPjIwMTctMDItMDg8L3RkPjx0ZCBjbGFzcz0idDAyIj4xNzAyMDg3OTwvdGQ%2BPHRkIGNsYXNzPSJkYjEgZnciPjAyIDA1IDA2IDEwIDA0PC90ZD48dGQgY2xhc3M9ImRiMSBmdyI%2BMDIgMDQgMDUgMDYgMTA8L3RkPjx0ZCBjbGFzcz0iYjA1Ij4xPC90ZD48dGQgY2xhc3M9ImIwNSBiYWxsX3JlZCI%2BMjwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjE8L3RkPjx0ZCBjbGFzcz0iYjA1IGJhbGxfcmVkIj40PC90ZD48dGQgY2xhc3M9ImIwNSBiYWxsX3JlZCI%2BNTwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQiPjY8L3RkPjx0ZCBjbGFzcz0iYjA1Ij4zPC90ZD48dGQgY2xhc3M9ImIwNSI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjM8L3RkPjx0ZCBjbGFzcz0iYjA1IGJhbGxfcmVkIj4xMDwvdGQ%2BPHRkIGNsYXNzPSJiMDUgZGIxIj4yPC90ZD48dGQgY2xhc3M9ImIwMiI%2BMjAzNjwvdGQ%2BPHRkIGNsYXNzPSJiMDIgcmVjdF9ibHVlIj4xPC90ZD48dGQgY2xhc3M9ImIwMiI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjk8L3RkPjx0ZCBjbGFzcz0iYjAyIj4zPC90ZD48dGQgY2xhc3M9ImIwMiBkYjEiPjI2PC90ZD48dGQgY2xhc3M9ImIwMSI%2BMjk1PC90ZD48dGQgY2xhc3M9ImIwMSI%2BNTwvdGQ%2BPHRkIGNsYXNzPSJiMDEgcmVjdF95ZWxsb3ciPjI8L3RkPjx0ZCBjbGFzcz0iYjAxIj4zPC90ZD48dGQgY2xhc3M9ImIwMSI%2BNzwvdGQ%2BPHRkIGNsYXNzPSJiMDEgZGIxIj45PC90ZD48dGQgY2xhc3M9ImIwMiI%2BOTI8L3RkPjx0ZCBjbGFzcz0iYjAyIj43PC90ZD48dGQgY2xhc3M9ImIwMiByZWN0X3JlZCI%2BMjwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjI8L3RkPjx0ZCBjbGFzcz0iYjAyIj4xNjwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjMwPC90ZD48dHI%2BDQo8dGQ%2BMjAxNy0wMi0wODwvdGQ%2BPHRkIGNsYXNzPSJ0MDIiPjE3MDIwODgwPC90ZD48dGQgY2xhc3M9ImRiMSBmdyI%2BMDQgMDEgMDcgMDUgMTE8L3RkPjx0ZCBjbGFzcz0iZGIxIGZ3Ij4wMSAwNCAwNSAwNyAxMTwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQiPjE8L3RkPjx0ZCBjbGFzcz0iYjA1Ij4xPC90ZD48dGQgY2xhc3M9ImIwNSI%2BMjwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQiPjQ8L3RkPjx0ZCBjbGFzcz0iYjA1IGJhbGxfcmVkIj41PC90ZD48dGQgY2xhc3M9ImIwNSI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQiPjc8L3RkPjx0ZCBjbGFzcz0iYjA1Ij4yPC90ZD48dGQgY2xhc3M9ImIwNSI%2BNDwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjE8L3RkPjx0ZCBjbGFzcz0iYjA1IGJhbGxfcmVkIGRiMSI%2BMTE8L3RkPjx0ZCBjbGFzcz0iYjAyIj4yMDM3PC90ZD48dGQgY2xhc3M9ImIwMiI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjI8L3RkPjx0ZCBjbGFzcz0iYjAyIj4xMDwvdGQ%2BPHRkIGNsYXNzPSJiMDIgcmVjdF9ibHVlIj40PC90ZD48dGQgY2xhc3M9ImIwMiBkYjEiPjI3PC90ZD48dGQgY2xhc3M9ImIwMSI%2BMjk2PC90ZD48dGQgY2xhc3M9ImIwMSI%2BNjwvdGQ%2BPHRkIGNsYXNzPSJiMDEgcmVjdF95ZWxsb3ciPjI8L3RkPjx0ZCBjbGFzcz0iYjAxIj40PC90ZD48dGQgY2xhc3M9ImIwMSI%2BODwvdGQ%2BPHRkIGNsYXNzPSJiMDEgZGIxIj4xMDwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjkzPC90ZD48dGQgY2xhc3M9ImIwMiI%2BODwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjE8L3RkPjx0ZCBjbGFzcz0iYjAyIj4zPC90ZD48dGQgY2xhc3M9ImIwMiByZWN0X3JlZCI%2BNDwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjMxPC90ZD48dHIgY2xhc3M9ImRiMiI%2BDQo8dGQ%2BMjAxNy0wMi0wODwvdGQ%2BPHRkIGNsYXNzPSJ0MDIiPjE3MDIwODgxPC90ZD48dGQgY2xhc3M9ImRiMSBmdyI%2BMDggMTEgMDIgMDcgMTA8L3RkPjx0ZCBjbGFzcz0iZGIxIGZ3Ij4wMiAwNyAwOCAxMCAxMTwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjE8L3RkPjx0ZCBjbGFzcz0iYjA1IGJhbGxfcmVkIj4yPC90ZD48dGQgY2xhc3M9ImIwNSI%2BMzwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjE8L3RkPjx0ZCBjbGFzcz0iYjA1Ij4xPC90ZD48dGQgY2xhc3M9ImIwNSI%2BMjwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQiPjc8L3RkPjx0ZCBjbGFzcz0iYjA1IGJhbGxfcmVkIj44PC90ZD48dGQgY2xhc3M9ImIwNSI%2BNTwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQiPjEwPC90ZD48dGQgY2xhc3M9ImIwNSBiYWxsX3JlZCBkYjEiPjExPC90ZD48dGQgY2xhc3M9ImIwMiI%2BMjAzODwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjI8L3RkPjx0ZCBjbGFzcz0iYjAyIHJlY3RfYmx1ZSI%2BMjwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjExPC90ZD48dGQgY2xhc3M9ImIwMiI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDIgZGIxIj4yODwvdGQ%2BPHRkIGNsYXNzPSJiMDEiPjI5NzwvdGQ%2BPHRkIGNsYXNzPSJiMDEiPjc8L3RkPjx0ZCBjbGFzcz0iYjAxIj4xPC90ZD48dGQgY2xhc3M9ImIwMSI%2BNTwvdGQ%2BPHRkIGNsYXNzPSJiMDEgcmVjdF95ZWxsb3ciPjQ8L3RkPjx0ZCBjbGFzcz0iYjAxIGRiMSI%2BMTE8L3RkPjx0ZCBjbGFzcz0iYjAyIj45NDwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjk8L3RkPjx0ZCBjbGFzcz0iYjAyIj4yPC90ZD48dGQgY2xhc3M9ImIwMiByZWN0X3JlZCI%2BMzwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjE8L3RkPjx0ZCBjbGFzcz0iYjAyIj4zMjwvdGQ%2BPHRyPg0KPHRkPjIwMTctMDItMDg8L3RkPjx0ZCBjbGFzcz0idDAyIj4xNzAyMDg4MjwvdGQ%2BPHRkIGNsYXNzPSJkYjEgZnciPjA2IDExIDA4IDA0IDA3PC90ZD48dGQgY2xhc3M9ImRiMSBmdyI%2BMDQgMDYgMDcgMDggMTE8L3RkPjx0ZCBjbGFzcz0iYjA1Ij4yPC90ZD48dGQgY2xhc3M9ImIwNSI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjQ8L3RkPjx0ZCBjbGFzcz0iYjA1IGJhbGxfcmVkIj40PC90ZD48dGQgY2xhc3M9ImIwNSI%2BMjwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQiPjY8L3RkPjx0ZCBjbGFzcz0iYjA1IGJhbGxfcmVkIj43PC90ZD48dGQgY2xhc3M9ImIwNSBiYWxsX3JlZCI%2BODwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjY8L3RkPjx0ZCBjbGFzcz0iYjA1Ij4xPC90ZD48dGQgY2xhc3M9ImIwNSBiYWxsX3JlZCBkYjEiPjExPC90ZD48dGQgY2xhc3M9ImIwMiI%2BMjAzOTwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjM8L3RkPjx0ZCBjbGFzcz0iYjAyIHJlY3RfYmx1ZSI%2BMjwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjEyPC90ZD48dGQgY2xhc3M9ImIwMiI%2BMjwvdGQ%2BPHRkIGNsYXNzPSJiMDIgZGIxIj4yOTwvdGQ%2BPHRkIGNsYXNzPSJiMDEiPjI5ODwvdGQ%2BPHRkIGNsYXNzPSJiMDEiPjg8L3RkPjx0ZCBjbGFzcz0iYjAxIj4yPC90ZD48dGQgY2xhc3M9ImIwMSI%2BNjwvdGQ%2BPHRkIGNsYXNzPSJiMDEgcmVjdF95ZWxsb3ciPjQ8L3RkPjx0ZCBjbGFzcz0iYjAxIGRiMSI%2BMTI8L3RkPjx0ZCBjbGFzcz0iYjAyIj45NTwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjEwPC90ZD48dGQgY2xhc3M9ImIwMiByZWN0X3JlZCI%2BMjwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjE8L3RkPjx0ZCBjbGFzcz0iYjAyIj4yPC90ZD48dGQgY2xhc3M9ImIwMiI%2BMzM8L3RkPjx0cj4NCjx0ZD4yMDE3LTAyLTA4PC90ZD48dGQgY2xhc3M9InQwMiI%2BMTcwMjA4ODM8L3RkPjx0ZCBjbGFzcz0iZGIxIGZ3Ij4wMyAwNiAxMCAwOSAwMjwvdGQ%2BPHRkIGNsYXNzPSJkYjEgZnciPjAyIDAzIDA2IDA5IDEwPC90ZD48dGQgY2xhc3M9ImIwNSI%2BMzwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQiPjI8L3RkPjx0ZCBjbGFzcz0iYjA1IGJhbGxfcmVkIj4zPC90ZD48dGQgY2xhc3M9ImIwNSI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjM8L3RkPjx0ZCBjbGFzcz0iYjA1IGJhbGxfcmVkIj42PC90ZD48dGQgY2xhc3M9ImIwNSI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjE8L3RkPjx0ZCBjbGFzcz0iYjA1IGJhbGxfcmVkIj45PC90ZD48dGQgY2xhc3M9ImIwNSBiYWxsX3JlZCI%2BMTA8L3RkPjx0ZCBjbGFzcz0iYjA1IGRiMSI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjIwNDA8L3RkPjx0ZCBjbGFzcz0iYjAyIj40PC90ZD48dGQgY2xhc3M9ImIwMiByZWN0X2JsdWUiPjI8L3RkPjx0ZCBjbGFzcz0iYjAyIj4xMzwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjM8L3RkPjx0ZCBjbGFzcz0iYjAyIGRiMSI%2BMzA8L3RkPjx0ZCBjbGFzcz0iYjAxIj4yOTk8L3RkPjx0ZCBjbGFzcz0iYjAxIj45PC90ZD48dGQgY2xhc3M9ImIwMSI%2BMzwvdGQ%2BPHRkIGNsYXNzPSJiMDEgcmVjdF95ZWxsb3ciPjM8L3RkPjx0ZCBjbGFzcz0iYjAxIj4xPC90ZD48dGQgY2xhc3M9ImIwMSBkYjEiPjEzPC90ZD48dGQgY2xhc3M9ImIwMiI%2BOTY8L3RkPjx0ZCBjbGFzcz0iYjAyIj4xMTwvdGQ%2BPHRkIGNsYXNzPSJiMDIgcmVjdF9yZWQiPjI8L3RkPjx0ZCBjbGFzcz0iYjAyIj4yPC90ZD48dGQgY2xhc3M9ImIwMiI%2BMzwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjM0PC90ZD48dHI%2BDQo8dGQ%2BMjAxNy0wMi0wODwvdGQ%2BPHRkIGNsYXNzPSJ0MDIiPjE3MDIwODg0PC90ZD48dGQgY2xhc3M9ImRiMSBmdyI%2BMDIgMDEgMDMgMTAgMDc8L3RkPjx0ZCBjbGFzcz0iZGIxIGZ3Ij4wMSAwMiAwMyAwNyAxMDwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQiPjE8L3RkPjx0ZCBjbGFzcz0iYjA1IGJhbGxfcmVkIj4yPC90ZD48dGQgY2xhc3M9ImIwNSBiYWxsX3JlZCI%2BMzwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjI8L3RkPjx0ZCBjbGFzcz0iYjA1Ij40PC90ZD48dGQgY2xhc3M9ImIwNSI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQiPjc8L3RkPjx0ZCBjbGFzcz0iYjA1Ij4yPC90ZD48dGQgY2xhc3M9ImIwNSI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQiPjEwPC90ZD48dGQgY2xhc3M9ImIwNSBkYjEiPjI8L3RkPjx0ZCBjbGFzcz0iYjAyIj4yMDQxPC90ZD48dGQgY2xhc3M9ImIwMiI%2BNTwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjE8L3RkPjx0ZCBjbGFzcz0iYjAyIHJlY3RfYmx1ZSI%2BMzwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjQ8L3RkPjx0ZCBjbGFzcz0iYjAyIGRiMSI%2BMzE8L3RkPjx0ZCBjbGFzcz0iYjAxIj4zMDA8L3RkPjx0ZCBjbGFzcz0iYjAxIj4xMDwvdGQ%2BPHRkIGNsYXNzPSJiMDEgcmVjdF95ZWxsb3ciPjI8L3RkPjx0ZCBjbGFzcz0iYjAxIj4xPC90ZD48dGQgY2xhc3M9ImIwMSI%2BMjwvdGQ%2BPHRkIGNsYXNzPSJiMDEgZGIxIj4xNDwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjk3PC90ZD48dGQgY2xhc3M9ImIwMiI%2BMTI8L3RkPjx0ZCBjbGFzcz0iYjAyIj4xPC90ZD48dGQgY2xhc3M9ImIwMiI%2BMzwvdGQ%2BPHRkIGNsYXNzPSJiMDIgcmVjdF9yZWQiPjQ8L3RkPjx0ZCBjbGFzcz0iYjAyIj4zNTwvdGQ%2BPHRyPg0KPHRkPjIwMTctMDItMDg8L3RkPjx0ZCBjbGFzcz0idDAyIj4xNzAyMDg4NTwvdGQ%2BPHRkIGNsYXNzPSJkYjEgZnciPjAxIDAzIDExIDA1IDEwPC90ZD48dGQgY2xhc3M9ImRiMSBmdyI%2BMDEgMDMgMDUgMTAgMTE8L3RkPjx0ZCBjbGFzcz0iYjA1IGJhbGxfcmVkIj4xPC90ZD48dGQgY2xhc3M9ImIwNSI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQiPjM8L3RkPjx0ZCBjbGFzcz0iYjA1Ij4zPC90ZD48dGQgY2xhc3M9ImIwNSBiYWxsX3JlZCI%2BNTwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjI8L3RkPjx0ZCBjbGFzcz0iYjA1Ij4xPC90ZD48dGQgY2xhc3M9ImIwNSI%2BMzwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjI8L3RkPjx0ZCBjbGFzcz0iYjA1IGJhbGxfcmVkIj4xMDwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQgZGIxIj4xMTwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjIwNDI8L3RkPjx0ZCBjbGFzcz0iYjAyIj42PC90ZD48dGQgY2xhc3M9ImIwMiI%2BMjwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjE8L3RkPjx0ZCBjbGFzcz0iYjAyIHJlY3RfYmx1ZSI%2BNDwvdGQ%2BPHRkIGNsYXNzPSJiMDIgZGIxIj4zMjwvdGQ%2BPHRkIGNsYXNzPSJiMDEiPjMwMTwvdGQ%2BPHRkIGNsYXNzPSJiMDEiPjExPC90ZD48dGQgY2xhc3M9ImIwMSByZWN0X3llbGxvdyI%2BMjwvdGQ%2BPHRkIGNsYXNzPSJiMDEiPjI8L3RkPjx0ZCBjbGFzcz0iYjAxIj4zPC90ZD48dGQgY2xhc3M9ImIwMSBkYjEiPjE1PC90ZD48dGQgY2xhc3M9ImIwMiI%2BOTg8L3RkPjx0ZCBjbGFzcz0iYjAyIj4xMzwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjI8L3RkPjx0ZCBjbGFzcz0iYjAyIj40PC90ZD48dGQgY2xhc3M9ImIwMiByZWN0X3JlZCI%2BNDwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjM2PC90ZD48dHIgY2xhc3M9ImRiMiI%2BDQo8dGQ%2BMjAxNy0wMi0wOTwvdGQ%2BPHRkIGNsYXNzPSJ0MDIiPjE3MDIwOTAxPC90ZD48dGQgY2xhc3M9ImRiMSBmdyI%2BMDcgMDUgMTEgMDMgMDk8L3RkPjx0ZCBjbGFzcz0iZGIxIGZ3Ij4wMyAwNSAwNyAwOSAxMTwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjE8L3RkPjx0ZCBjbGFzcz0iYjA1Ij4yPC90ZD48dGQgY2xhc3M9ImIwNSBiYWxsX3JlZCI%2BMzwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjQ8L3RkPjx0ZCBjbGFzcz0iYjA1IGJhbGxfcmVkIj41PC90ZD48dGQgY2xhc3M9ImIwNSI%2BMzwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQiPjc8L3RkPjx0ZCBjbGFzcz0iYjA1Ij40PC90ZD48dGQgY2xhc3M9ImIwNSBiYWxsX3JlZCI%2BOTwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjE8L3RkPjx0ZCBjbGFzcz0iYjA1IGJhbGxfcmVkIGRiMSI%2BMTE8L3RkPjx0ZCBjbGFzcz0iYjAyIj4yMDQzPC90ZD48dGQgY2xhc3M9ImIwMiI%2BNzwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjM8L3RkPjx0ZCBjbGFzcz0iYjAyIj4yPC90ZD48dGQgY2xhc3M9ImIwMiI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDIgcmVjdF9ibHVlIGRiMSI%2BNTwvdGQ%2BPHRkIGNsYXNzPSJiMDEiPjMwMjwvdGQ%2BPHRkIGNsYXNzPSJiMDEiPjEyPC90ZD48dGQgY2xhc3M9ImIwMSI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDEgcmVjdF95ZWxsb3ciPjM8L3RkPjx0ZCBjbGFzcz0iYjAxIj40PC90ZD48dGQgY2xhc3M9ImIwMSBkYjEiPjE2PC90ZD48dGQgY2xhc3M9ImIwMiI%2BOTk8L3RkPjx0ZCBjbGFzcz0iYjAyIj4xNDwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjM8L3RkPjx0ZCBjbGFzcz0iYjAyIj41PC90ZD48dGQgY2xhc3M9ImIwMiByZWN0X3JlZCI%2BNDwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjM3PC90ZD48dHI%2BDQo8dGQ%2BMjAxNy0wMi0wOTwvdGQ%2BPHRkIGNsYXNzPSJ0MDIiPjE3MDIwOTAyPC90ZD48dGQgY2xhc3M9ImRiMSBmdyI%2BMDEgMTAgMDcgMDQgMDI8L3RkPjx0ZCBjbGFzcz0iZGIxIGZ3Ij4wMSAwMiAwNCAwNyAxMDwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQiPjE8L3RkPjx0ZCBjbGFzcz0iYjA1IGJhbGxfcmVkIj4yPC90ZD48dGQgY2xhc3M9ImIwNSI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQiPjQ8L3RkPjx0ZCBjbGFzcz0iYjA1Ij4xPC90ZD48dGQgY2xhc3M9ImIwNSI%2BNDwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQiPjc8L3RkPjx0ZCBjbGFzcz0iYjA1Ij41PC90ZD48dGQgY2xhc3M9ImIwNSI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQiPjEwPC90ZD48dGQgY2xhc3M9ImIwNSBkYjEiPjE8L3RkPjx0ZCBjbGFzcz0iYjAyIj4yMDQ0PC90ZD48dGQgY2xhc3M9ImIwMiI%2BODwvdGQ%2BPHRkIGNsYXNzPSJiMDIgcmVjdF9ibHVlIj4yPC90ZD48dGQgY2xhc3M9ImIwMiI%2BMzwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjI8L3RkPjx0ZCBjbGFzcz0iYjAyIGRiMSI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDEiPjMwMzwvdGQ%2BPHRkIGNsYXNzPSJiMDEiPjEzPC90ZD48dGQgY2xhc3M9ImIwMSByZWN0X3llbGxvdyI%2BMjwvdGQ%2BPHRkIGNsYXNzPSJiMDEiPjE8L3RkPjx0ZCBjbGFzcz0iYjAxIj41PC90ZD48dGQgY2xhc3M9ImIwMSBkYjEiPjE3PC90ZD48dGQgY2xhc3M9ImIwMiI%2BMTAwPC90ZD48dGQgY2xhc3M9ImIwMiI%2BMTU8L3RkPjx0ZCBjbGFzcz0iYjAyIj40PC90ZD48dGQgY2xhc3M9ImIwMiByZWN0X3JlZCI%2BMzwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjE8L3RkPjx0ZCBjbGFzcz0iYjAyIj4zODwvdGQ%2BPHRyPg0KPHRkPjIwMTctMDItMDk8L3RkPjx0ZCBjbGFzcz0idDAyIj4xNzAyMDkwMzwvdGQ%2BPHRkIGNsYXNzPSJkYjEgZnciPjAyIDAxIDA2IDExIDEwPC90ZD48dGQgY2xhc3M9ImRiMSBmdyI%2BMDEgMDIgMDYgMTAgMTE8L3RkPjx0ZCBjbGFzcz0iYjA1IGJhbGxfcmVkIj4xPC90ZD48dGQgY2xhc3M9ImIwNSBiYWxsX3JlZCI%2BMjwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjI8L3RkPjx0ZCBjbGFzcz0iYjA1Ij4xPC90ZD48dGQgY2xhc3M9ImIwNSI%2BMjwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQiPjY8L3RkPjx0ZCBjbGFzcz0iYjA1Ij4xPC90ZD48dGQgY2xhc3M9ImIwNSI%2BNjwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjI8L3RkPjx0ZCBjbGFzcz0iYjA1IGJhbGxfcmVkIj4xMDwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQgZGIxIj4xMTwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjIwNDU8L3RkPjx0ZCBjbGFzcz0iYjAyIj45PC90ZD48dGQgY2xhc3M9ImIwMiByZWN0X2JsdWUiPjI8L3RkPjx0ZCBjbGFzcz0iYjAyIj40PC90ZD48dGQgY2xhc3M9ImIwMiI%2BMzwvdGQ%2BPHRkIGNsYXNzPSJiMDIgZGIxIj4yPC90ZD48dGQgY2xhc3M9ImIwMSI%2BMzA0PC90ZD48dGQgY2xhc3M9ImIwMSI%2BMTQ8L3RkPjx0ZCBjbGFzcz0iYjAxIj4xPC90ZD48dGQgY2xhc3M9ImIwMSByZWN0X3llbGxvdyI%2BMzwvdGQ%2BPHRkIGNsYXNzPSJiMDEiPjY8L3RkPjx0ZCBjbGFzcz0iYjAxIGRiMSI%2BMTg8L3RkPjx0ZCBjbGFzcz0iYjAyIj4xMDE8L3RkPjx0ZCBjbGFzcz0iYjAyIj4xNjwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjU8L3RkPjx0ZCBjbGFzcz0iYjAyIHJlY3RfcmVkIj4zPC90ZD48dGQgY2xhc3M9ImIwMiI%2BMjwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjM5PC90ZD48dHI%2BDQo8dGQ%2BMjAxNy0wMi0wOTwvdGQ%2BPHRkIGNsYXNzPSJ0MDIiPjE3MDIwOTA0PC90ZD48dGQgY2xhc3M9ImRiMSBmdyI%2BMDggMDkgMDEgMTEgMDI8L3RkPjx0ZCBjbGFzcz0iZGIxIGZ3Ij4wMSAwMiAwOCAwOSAxMTwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQiPjE8L3RkPjx0ZCBjbGFzcz0iYjA1IGJhbGxfcmVkIj4yPC90ZD48dGQgY2xhc3M9ImIwNSI%2BMzwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjI8L3RkPjx0ZCBjbGFzcz0iYjA1Ij4zPC90ZD48dGQgY2xhc3M9ImIwNSI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjI8L3RkPjx0ZCBjbGFzcz0iYjA1IGJhbGxfcmVkIj44PC90ZD48dGQgY2xhc3M9ImIwNSBiYWxsX3JlZCI%2BOTwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjE8L3RkPjx0ZCBjbGFzcz0iYjA1IGJhbGxfcmVkIGRiMSI%2BMTE8L3RkPjx0ZCBjbGFzcz0iYjAyIj4yMDQ2PC90ZD48dGQgY2xhc3M9ImIwMiI%2BMTA8L3RkPjx0ZCBjbGFzcz0iYjAyIj4xPC90ZD48dGQgY2xhc3M9ImIwMiByZWN0X2JsdWUiPjM8L3RkPjx0ZCBjbGFzcz0iYjAyIj40PC90ZD48dGQgY2xhc3M9ImIwMiBkYjEiPjM8L3RkPjx0ZCBjbGFzcz0iYjAxIj4zMDU8L3RkPjx0ZCBjbGFzcz0iYjAxIj4xNTwvdGQ%2BPHRkIGNsYXNzPSJiMDEiPjI8L3RkPjx0ZCBjbGFzcz0iYjAxIHJlY3RfeWVsbG93Ij4zPC90ZD48dGQgY2xhc3M9ImIwMSI%2BNzwvdGQ%2BPHRkIGNsYXNzPSJiMDEgZGIxIj4xOTwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjEwMjwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjE3PC90ZD48dGQgY2xhc3M9ImIwMiI%2BNjwvdGQ%2BPHRkIGNsYXNzPSJiMDIgcmVjdF9yZWQiPjM8L3RkPjx0ZCBjbGFzcz0iYjAyIj4zPC90ZD48dGQgY2xhc3M9ImIwMiI%2BNDA8L3RkPjx0cj4NCjx0ZD4yMDE3LTAyLTA5PC90ZD48dGQgY2xhc3M9InQwMiI%2BMTcwMjA5MDU8L3RkPjx0ZCBjbGFzcz0iZGIxIGZ3Ij4wMiAxMCAwNSAwNiAwOTwvdGQ%2BPHRkIGNsYXNzPSJkYjEgZnciPjAyIDA1IDA2IDA5IDEwPC90ZD48dGQgY2xhc3M9ImIwNSI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQiPjI8L3RkPjx0ZCBjbGFzcz0iYjA1Ij40PC90ZD48dGQgY2xhc3M9ImIwNSI%2BMzwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQiPjU8L3RkPjx0ZCBjbGFzcz0iYjA1IGJhbGxfcmVkIj42PC90ZD48dGQgY2xhc3M9ImIwNSI%2BMzwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjE8L3RkPjx0ZCBjbGFzcz0iYjA1IGJhbGxfcmVkIj45PC90ZD48dGQgY2xhc3M9ImIwNSBiYWxsX3JlZCI%2BMTA8L3RkPjx0ZCBjbGFzcz0iYjA1IGRiMSI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjIwNDc8L3RkPjx0ZCBjbGFzcz0iYjAyIj4xMTwvdGQ%2BPHRkIGNsYXNzPSJiMDIgcmVjdF9ibHVlIj4yPC90ZD48dGQgY2xhc3M9ImIwMiI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjU8L3RkPjx0ZCBjbGFzcz0iYjAyIGRiMSI%2BNDwvdGQ%2BPHRkIGNsYXNzPSJiMDEiPjMwNjwvdGQ%2BPHRkIGNsYXNzPSJiMDEiPjE2PC90ZD48dGQgY2xhc3M9ImIwMSI%2BMzwvdGQ%2BPHRkIGNsYXNzPSJiMDEgcmVjdF95ZWxsb3ciPjM8L3RkPjx0ZCBjbGFzcz0iYjAxIj44PC90ZD48dGQgY2xhc3M9ImIwMSBkYjEiPjIwPC90ZD48dGQgY2xhc3M9ImIwMiI%2BMTAzPC90ZD48dGQgY2xhc3M9ImIwMiI%2BMTg8L3RkPjx0ZCBjbGFzcz0iYjAyIHJlY3RfcmVkIj4yPC90ZD48dGQgY2xhc3M9ImIwMiI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjQ8L3RkPjx0ZCBjbGFzcz0iYjAyIj40MTwvdGQ%2BPHRyIGNsYXNzPSJkYjIiPg0KPHRkPjIwMTctMDItMDk8L3RkPjx0ZCBjbGFzcz0idDAyIj4xNzAyMDkwNjwvdGQ%2BPHRkIGNsYXNzPSJkYjEgZnciPjA2IDA0IDExIDAzIDAxPC90ZD48dGQgY2xhc3M9ImRiMSBmdyI%2BMDEgMDMgMDQgMDYgMTE8L3RkPjx0ZCBjbGFzcz0iYjA1IGJhbGxfcmVkIj4xPC90ZD48dGQgY2xhc3M9ImIwNSI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQiPjM8L3RkPjx0ZCBjbGFzcz0iYjA1IGJhbGxfcmVkIj40PC90ZD48dGQgY2xhc3M9ImIwNSI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQiPjY8L3RkPjx0ZCBjbGFzcz0iYjA1Ij40PC90ZD48dGQgY2xhc3M9ImIwNSI%2BMjwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjE8L3RkPjx0ZCBjbGFzcz0iYjA1Ij4xPC90ZD48dGQgY2xhc3M9ImIwNSBiYWxsX3JlZCBkYjEiPjExPC90ZD48dGQgY2xhc3M9ImIwMiI%2BMjA0ODwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjEyPC90ZD48dGQgY2xhc3M9ImIwMiI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDIgcmVjdF9ibHVlIj4zPC90ZD48dGQgY2xhc3M9ImIwMiI%2BNjwvdGQ%2BPHRkIGNsYXNzPSJiMDIgZGIxIj41PC90ZD48dGQgY2xhc3M9ImIwMSI%2BMzA3PC90ZD48dGQgY2xhc3M9ImIwMSI%2BMTc8L3RkPjx0ZCBjbGFzcz0iYjAxIHJlY3RfeWVsbG93Ij4yPC90ZD48dGQgY2xhc3M9ImIwMSI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDEiPjk8L3RkPjx0ZCBjbGFzcz0iYjAxIGRiMSI%2BMjE8L3RkPjx0ZCBjbGFzcz0iYjAyIj4xMDQ8L3RkPjx0ZCBjbGFzcz0iYjAyIj4xOTwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjE8L3RkPjx0ZCBjbGFzcz0iYjAyIHJlY3RfcmVkIj4zPC90ZD48dGQgY2xhc3M9ImIwMiI%2BNTwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjQyPC90ZD48dHI%2BDQo8dGQ%2BMjAxNy0wMi0wOTwvdGQ%2BPHRkIGNsYXNzPSJ0MDIiPjE3MDIwOTA3PC90ZD48dGQgY2xhc3M9ImRiMSBmdyI%2BMTEgMDMgMDcgMDkgMDQ8L3RkPjx0ZCBjbGFzcz0iZGIxIGZ3Ij4wMyAwNCAwNyAwOSAxMTwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjE8L3RkPjx0ZCBjbGFzcz0iYjA1Ij4yPC90ZD48dGQgY2xhc3M9ImIwNSBiYWxsX3JlZCI%2BMzwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQiPjQ8L3RkPjx0ZCBjbGFzcz0iYjA1Ij4yPC90ZD48dGQgY2xhc3M9ImIwNSI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQiPjc8L3RkPjx0ZCBjbGFzcz0iYjA1Ij4zPC90ZD48dGQgY2xhc3M9ImIwNSBiYWxsX3JlZCI%2BOTwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjI8L3RkPjx0ZCBjbGFzcz0iYjA1IGJhbGxfcmVkIGRiMSI%2BMTE8L3RkPjx0ZCBjbGFzcz0iYjAyIj4yMDQ5PC90ZD48dGQgY2xhc3M9ImIwMiI%2BMTM8L3RkPjx0ZCBjbGFzcz0iYjAyIj4yPC90ZD48dGQgY2xhc3M9ImIwMiI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDIgcmVjdF9ibHVlIj40PC90ZD48dGQgY2xhc3M9ImIwMiBkYjEiPjY8L3RkPjx0ZCBjbGFzcz0iYjAxIj4zMDg8L3RkPjx0ZCBjbGFzcz0iYjAxIj4xODwvdGQ%2BPHRkIGNsYXNzPSJiMDEiPjE8L3RkPjx0ZCBjbGFzcz0iYjAxIHJlY3RfeWVsbG93Ij4zPC90ZD48dGQgY2xhc3M9ImIwMSI%2BMTA8L3RkPjx0ZCBjbGFzcz0iYjAxIGRiMSI%2BMjI8L3RkPjx0ZCBjbGFzcz0iYjAyIj4xMDU8L3RkPjx0ZCBjbGFzcz0iYjAyIj4yMDwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjI8L3RkPjx0ZCBjbGFzcz0iYjAyIHJlY3RfcmVkIj4zPC90ZD48dGQgY2xhc3M9ImIwMiI%2BNjwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjQzPC90ZD48dHI%2BDQo8dGQ%2BMjAxNy0wMi0wOTwvdGQ%2BPHRkIGNsYXNzPSJ0MDIiPjE3MDIwOTA4PC90ZD48dGQgY2xhc3M9ImRiMSBmdyI%2BMDkgMDMgMDUgMTEgMTA8L3RkPjx0ZCBjbGFzcz0iZGIxIGZ3Ij4wMyAwNSAwOSAxMCAxMTwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjI8L3RkPjx0ZCBjbGFzcz0iYjA1Ij4zPC90ZD48dGQgY2xhc3M9ImIwNSBiYWxsX3JlZCI%2BMzwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjE8L3RkPjx0ZCBjbGFzcz0iYjA1IGJhbGxfcmVkIj41PC90ZD48dGQgY2xhc3M9ImIwNSI%2BMjwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjE8L3RkPjx0ZCBjbGFzcz0iYjA1Ij40PC90ZD48dGQgY2xhc3M9ImIwNSBiYWxsX3JlZCI%2BOTwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQiPjEwPC90ZD48dGQgY2xhc3M9ImIwNSBiYWxsX3JlZCBkYjEiPjExPC90ZD48dGQgY2xhc3M9ImIwMiI%2BMjA1MDwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjE0PC90ZD48dGQgY2xhc3M9ImIwMiI%2BMzwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjI8L3RkPjx0ZCBjbGFzcz0iYjAyIHJlY3RfYmx1ZSI%2BNDwvdGQ%2BPHRkIGNsYXNzPSJiMDIgZGIxIj43PC90ZD48dGQgY2xhc3M9ImIwMSI%2BMzA5PC90ZD48dGQgY2xhc3M9ImIwMSI%2BMTk8L3RkPjx0ZCBjbGFzcz0iYjAxIj4yPC90ZD48dGQgY2xhc3M9ImIwMSByZWN0X3llbGxvdyI%2BMzwvdGQ%2BPHRkIGNsYXNzPSJiMDEiPjExPC90ZD48dGQgY2xhc3M9ImIwMSBkYjEiPjIzPC90ZD48dGQgY2xhc3M9ImIwMiI%2BMTA2PC90ZD48dGQgY2xhc3M9ImIwMiI%2BMjE8L3RkPjx0ZCBjbGFzcz0iYjAyIj4zPC90ZD48dGQgY2xhc3M9ImIwMiByZWN0X3JlZCI%2BMzwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjc8L3RkPjx0ZCBjbGFzcz0iYjAyIj40NDwvdGQ%2BPHRyPg0KPHRkPjIwMTctMDItMDk8L3RkPjx0ZCBjbGFzcz0idDAyIj4xNzAyMDkwOTwvdGQ%2BPHRkIGNsYXNzPSJkYjEgZnciPjA3IDExIDEwIDAzIDA0PC90ZD48dGQgY2xhc3M9ImRiMSBmdyI%2BMDMgMDQgMDcgMTAgMTE8L3RkPjx0ZCBjbGFzcz0iYjA1Ij4zPC90ZD48dGQgY2xhc3M9ImIwNSI%2BNDwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQiPjM8L3RkPjx0ZCBjbGFzcz0iYjA1IGJhbGxfcmVkIj40PC90ZD48dGQgY2xhc3M9ImIwNSI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjM8L3RkPjx0ZCBjbGFzcz0iYjA1IGJhbGxfcmVkIj43PC90ZD48dGQgY2xhc3M9ImIwNSI%2BNTwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjE8L3RkPjx0ZCBjbGFzcz0iYjA1IGJhbGxfcmVkIj4xMDwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQgZGIxIj4xMTwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjIwNTE8L3RkPjx0ZCBjbGFzcz0iYjAyIj4xNTwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjQ8L3RkPjx0ZCBjbGFzcz0iYjAyIHJlY3RfYmx1ZSI%2BMzwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjE8L3RkPjx0ZCBjbGFzcz0iYjAyIGRiMSI%2BODwvdGQ%2BPHRkIGNsYXNzPSJiMDEiPjMxMDwvdGQ%2BPHRkIGNsYXNzPSJiMDEiPjIwPC90ZD48dGQgY2xhc3M9ImIwMSI%2BMzwvdGQ%2BPHRkIGNsYXNzPSJiMDEgcmVjdF95ZWxsb3ciPjM8L3RkPjx0ZCBjbGFzcz0iYjAxIj4xMjwvdGQ%2BPHRkIGNsYXNzPSJiMDEgZGIxIj4yNDwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjEwNzwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjIyPC90ZD48dGQgY2xhc3M9ImIwMiI%2BNDwvdGQ%2BPHRkIGNsYXNzPSJiMDIgcmVjdF9yZWQiPjM8L3RkPjx0ZCBjbGFzcz0iYjAyIj44PC90ZD48dGQgY2xhc3M9ImIwMiI%2BNDU8L3RkPjx0cj4NCjx0ZD4yMDE3LTAyLTA5PC90ZD48dGQgY2xhc3M9InQwMiI%2BMTcwMjA5MTA8L3RkPjx0ZCBjbGFzcz0iZGIxIGZ3Ij4xMSAwOCAwNSAwMyAxMDwvdGQ%2BPHRkIGNsYXNzPSJkYjEgZnciPjAzIDA1IDA4IDEwIDExPC90ZD48dGQgY2xhc3M9ImIwNSI%2BNDwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjU8L3RkPjx0ZCBjbGFzcz0iYjA1IGJhbGxfcmVkIj4zPC90ZD48dGQgY2xhc3M9ImIwNSI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQiPjU8L3RkPjx0ZCBjbGFzcz0iYjA1Ij40PC90ZD48dGQgY2xhc3M9ImIwNSI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQiPjg8L3RkPjx0ZCBjbGFzcz0iYjA1Ij4yPC90ZD48dGQgY2xhc3M9ImIwNSBiYWxsX3JlZCI%2BMTA8L3RkPjx0ZCBjbGFzcz0iYjA1IGJhbGxfcmVkIGRiMSI%2BMTE8L3RkPjx0ZCBjbGFzcz0iYjAyIj4yMDUyPC90ZD48dGQgY2xhc3M9ImIwMiI%2BMTY8L3RkPjx0ZCBjbGFzcz0iYjAyIj41PC90ZD48dGQgY2xhc3M9ImIwMiByZWN0X2JsdWUiPjM8L3RkPjx0ZCBjbGFzcz0iYjAyIj4yPC90ZD48dGQgY2xhc3M9ImIwMiBkYjEiPjk8L3RkPjx0ZCBjbGFzcz0iYjAxIj4zMTE8L3RkPjx0ZCBjbGFzcz0iYjAxIj4yMTwvdGQ%2BPHRkIGNsYXNzPSJiMDEiPjQ8L3RkPjx0ZCBjbGFzcz0iYjAxIHJlY3RfeWVsbG93Ij4zPC90ZD48dGQgY2xhc3M9ImIwMSI%2BMTM8L3RkPjx0ZCBjbGFzcz0iYjAxIGRiMSI%2BMjU8L3RkPjx0ZCBjbGFzcz0iYjAyIj4xMDg8L3RkPjx0ZCBjbGFzcz0iYjAyIj4yMzwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjU8L3RkPjx0ZCBjbGFzcz0iYjAyIHJlY3RfcmVkIj4zPC90ZD48dGQgY2xhc3M9ImIwMiI%2BOTwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjQ2PC90ZD48dHIgY2xhc3M9ImRiMiI%2BDQo8dGQ%2BMjAxNy0wMi0wOTwvdGQ%2BPHRkIGNsYXNzPSJ0MDIiPjE3MDIwOTExPC90ZD48dGQgY2xhc3M9ImRiMSBmdyI%2BMDYgMDEgMTEgMDkgMTA8L3RkPjx0ZCBjbGFzcz0iZGIxIGZ3Ij4wMSAwNiAwOSAxMCAxMTwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQiPjE8L3RkPjx0ZCBjbGFzcz0iYjA1Ij42PC90ZD48dGQgY2xhc3M9ImIwNSI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjI8L3RkPjx0ZCBjbGFzcz0iYjA1Ij4xPC90ZD48dGQgY2xhc3M9ImIwNSBiYWxsX3JlZCI%2BNjwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjI8L3RkPjx0ZCBjbGFzcz0iYjA1Ij4xPC90ZD48dGQgY2xhc3M9ImIwNSBiYWxsX3JlZCI%2BOTwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQiPjEwPC90ZD48dGQgY2xhc3M9ImIwNSBiYWxsX3JlZCBkYjEiPjExPC90ZD48dGQgY2xhc3M9ImIwMiI%2BMjA1MzwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjE3PC90ZD48dGQgY2xhc3M9ImIwMiI%2BNjwvdGQ%2BPHRkIGNsYXNzPSJiMDIgcmVjdF9ibHVlIj4zPC90ZD48dGQgY2xhc3M9ImIwMiI%2BMzwvdGQ%2BPHRkIGNsYXNzPSJiMDIgZGIxIj4xMDwvdGQ%2BPHRkIGNsYXNzPSJiMDEiPjMxMjwvdGQ%2BPHRkIGNsYXNzPSJiMDEiPjIyPC90ZD48dGQgY2xhc3M9ImIwMSI%2BNTwvdGQ%2BPHRkIGNsYXNzPSJiMDEiPjE8L3RkPjx0ZCBjbGFzcz0iYjAxIHJlY3RfeWVsbG93Ij40PC90ZD48dGQgY2xhc3M9ImIwMSBkYjEiPjI2PC90ZD48dGQgY2xhc3M9ImIwMiI%2BMTA5PC90ZD48dGQgY2xhc3M9ImIwMiI%2BMjQ8L3RkPjx0ZCBjbGFzcz0iYjAyIHJlY3RfcmVkIj4yPC90ZD48dGQgY2xhc3M9ImIwMiI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjEwPC90ZD48dGQgY2xhc3M9ImIwMiI%2BNDc8L3RkPjx0cj4NCjx0ZD4yMDE3LTAyLTA5PC90ZD48dGQgY2xhc3M9InQwMiI%2BMTcwMjA5MTI8L3RkPjx0ZCBjbGFzcz0iZGIxIGZ3Ij4wNCAwMSAxMSAwNyAwODwvdGQ%2BPHRkIGNsYXNzPSJkYjEgZnciPjAxIDA0IDA3IDA4IDExPC90ZD48dGQgY2xhc3M9ImIwNSBiYWxsX3JlZCI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjc8L3RkPjx0ZCBjbGFzcz0iYjA1Ij4yPC90ZD48dGQgY2xhc3M9ImIwNSBiYWxsX3JlZCI%2BNDwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjI8L3RkPjx0ZCBjbGFzcz0iYjA1Ij4xPC90ZD48dGQgY2xhc3M9ImIwNSBiYWxsX3JlZCI%2BNzwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQiPjg8L3RkPjx0ZCBjbGFzcz0iYjA1Ij4xPC90ZD48dGQgY2xhc3M9ImIwNSI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQgZGIxIj4xMTwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjIwNTQ8L3RkPjx0ZCBjbGFzcz0iYjAyIj4xODwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjc8L3RkPjx0ZCBjbGFzcz0iYjAyIHJlY3RfYmx1ZSI%2BMzwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjQ8L3RkPjx0ZCBjbGFzcz0iYjAyIGRiMSI%2BMTE8L3RkPjx0ZCBjbGFzcz0iYjAxIj4zMTM8L3RkPjx0ZCBjbGFzcz0iYjAxIj4yMzwvdGQ%2BPHRkIGNsYXNzPSJiMDEiPjY8L3RkPjx0ZCBjbGFzcz0iYjAxIHJlY3RfeWVsbG93Ij4zPC90ZD48dGQgY2xhc3M9ImIwMSI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDEgZGIxIj4yNzwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjExMDwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjI1PC90ZD48dGQgY2xhc3M9ImIwMiI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDIgcmVjdF9yZWQiPjM8L3RkPjx0ZCBjbGFzcz0iYjAyIj4xMTwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjQ4PC90ZD48dHI%2BDQo8dGQ%2BMjAxNy0wMi0wOTwvdGQ%2BPHRkIGNsYXNzPSJ0MDIiPjE3MDIwOTEzPC90ZD48dGQgY2xhc3M9ImRiMSBmdyI%2BMTEgMDUgMDIgMDYgMDQ8L3RkPjx0ZCBjbGFzcz0iZGIxIGZ3Ij4wMiAwNCAwNSAwNiAxMTwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjE8L3RkPjx0ZCBjbGFzcz0iYjA1IGJhbGxfcmVkIj4yPC90ZD48dGQgY2xhc3M9ImIwNSI%2BMzwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQiPjQ8L3RkPjx0ZCBjbGFzcz0iYjA1IGJhbGxfcmVkIj41PC90ZD48dGQgY2xhc3M9ImIwNSBiYWxsX3JlZCI%2BNjwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjE8L3RkPjx0ZCBjbGFzcz0iYjA1Ij4xPC90ZD48dGQgY2xhc3M9ImIwNSI%2BMjwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjI8L3RkPjx0ZCBjbGFzcz0iYjA1IGJhbGxfcmVkIGRiMSI%2BMTE8L3RkPjx0ZCBjbGFzcz0iYjAyIj4yMDU1PC90ZD48dGQgY2xhc3M9ImIwMiI%2BMTk8L3RkPjx0ZCBjbGFzcz0iYjAyIHJlY3RfYmx1ZSI%2BMjwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjE8L3RkPjx0ZCBjbGFzcz0iYjAyIj41PC90ZD48dGQgY2xhc3M9ImIwMiBkYjEiPjEyPC90ZD48dGQgY2xhc3M9ImIwMSI%2BMzE0PC90ZD48dGQgY2xhc3M9ImIwMSI%2BMjQ8L3RkPjx0ZCBjbGFzcz0iYjAxIHJlY3RfeWVsbG93Ij4yPC90ZD48dGQgY2xhc3M9ImIwMSI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDEiPjI8L3RkPjx0ZCBjbGFzcz0iYjAxIGRiMSI%2BMjg8L3RkPjx0ZCBjbGFzcz0iYjAyIj4xMTE8L3RkPjx0ZCBjbGFzcz0iYjAyIj4yNjwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjI8L3RkPjx0ZCBjbGFzcz0iYjAyIHJlY3RfcmVkIj4zPC90ZD48dGQgY2xhc3M9ImIwMiI%2BMTI8L3RkPjx0ZCBjbGFzcz0iYjAyIj40OTwvdGQ%2BPHRyPg0KPHRkPjIwMTctMDItMDk8L3RkPjx0ZCBjbGFzcz0idDAyIj4xNzAyMDkxNDwvdGQ%2BPHRkIGNsYXNzPSJkYjEgZnciPjA4IDExIDAzIDA3IDEwPC90ZD48dGQgY2xhc3M9ImRiMSBmdyI%2BMDMgMDcgMDggMTAgMTE8L3RkPjx0ZCBjbGFzcz0iYjA1Ij4yPC90ZD48dGQgY2xhc3M9ImIwNSI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQiPjM8L3RkPjx0ZCBjbGFzcz0iYjA1Ij4xPC90ZD48dGQgY2xhc3M9ImIwNSI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjE8L3RkPjx0ZCBjbGFzcz0iYjA1IGJhbGxfcmVkIj43PC90ZD48dGQgY2xhc3M9ImIwNSBiYWxsX3JlZCI%2BODwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjM8L3RkPjx0ZCBjbGFzcz0iYjA1IGJhbGxfcmVkIj4xMDwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQgZGIxIj4xMTwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjIwNTY8L3RkPjx0ZCBjbGFzcz0iYjAyIj4yMDwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjE8L3RkPjx0ZCBjbGFzcz0iYjAyIHJlY3RfYmx1ZSI%2BMzwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjY8L3RkPjx0ZCBjbGFzcz0iYjAyIGRiMSI%2BMTM8L3RkPjx0ZCBjbGFzcz0iYjAxIj4zMTU8L3RkPjx0ZCBjbGFzcz0iYjAxIj4yNTwvdGQ%2BPHRkIGNsYXNzPSJiMDEiPjE8L3RkPjx0ZCBjbGFzcz0iYjAxIj4yPC90ZD48dGQgY2xhc3M9ImIwMSByZWN0X3llbGxvdyI%2BNDwvdGQ%2BPHRkIGNsYXNzPSJiMDEgZGIxIj4yOTwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjExMjwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjI3PC90ZD48dGQgY2xhc3M9ImIwMiI%2BMzwvdGQ%2BPHRkIGNsYXNzPSJiMDIgcmVjdF9yZWQiPjM8L3RkPjx0ZCBjbGFzcz0iYjAyIj4xMzwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjUwPC90ZD48dHI%2BDQo8dGQ%2BMjAxNy0wMi0wOTwvdGQ%2BPHRkIGNsYXNzPSJ0MDIiPjE3MDIwOTE1PC90ZD48dGQgY2xhc3M9ImRiMSBmdyI%2BMDYgMDIgMDQgMTEgMDg8L3RkPjx0ZCBjbGFzcz0iZGIxIGZ3Ij4wMiAwNCAwNiAwOCAxMTwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjM8L3RkPjx0ZCBjbGFzcz0iYjA1IGJhbGxfcmVkIj4yPC90ZD48dGQgY2xhc3M9ImIwNSI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQiPjQ8L3RkPjx0ZCBjbGFzcz0iYjA1Ij4yPC90ZD48dGQgY2xhc3M9ImIwNSBiYWxsX3JlZCI%2BNjwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjE8L3RkPjx0ZCBjbGFzcz0iYjA1IGJhbGxfcmVkIj44PC90ZD48dGQgY2xhc3M9ImIwNSI%2BNDwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjE8L3RkPjx0ZCBjbGFzcz0iYjA1IGJhbGxfcmVkIGRiMSI%2BMTE8L3RkPjx0ZCBjbGFzcz0iYjAyIj4yMDU3PC90ZD48dGQgY2xhc3M9ImIwMiByZWN0X2JsdWUiPjE8L3RkPjx0ZCBjbGFzcz0iYjAyIj4yPC90ZD48dGQgY2xhc3M9ImIwMiI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjc8L3RkPjx0ZCBjbGFzcz0iYjAyIGRiMSI%2BMTQ8L3RkPjx0ZCBjbGFzcz0iYjAxIj4zMTY8L3RkPjx0ZCBjbGFzcz0iYjAxIj4yNjwvdGQ%2BPHRkIGNsYXNzPSJiMDEiPjI8L3RkPjx0ZCBjbGFzcz0iYjAxIHJlY3RfeWVsbG93Ij4zPC90ZD48dGQgY2xhc3M9ImIwMSI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDEgZGIxIj4zMDwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjExMzwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjI4PC90ZD48dGQgY2xhc3M9ImIwMiByZWN0X3JlZCI%2BMjwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjE8L3RkPjx0ZCBjbGFzcz0iYjAyIj4xNDwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjUxPC90ZD48dHIgY2xhc3M9ImRiMiI%2BDQo8dGQ%2BMjAxNy0wMi0wOTwvdGQ%2BPHRkIGNsYXNzPSJ0MDIiPjE3MDIwOTE2PC90ZD48dGQgY2xhc3M9ImRiMSBmdyI%2BMDEgMDggMDkgMDIgMDc8L3RkPjx0ZCBjbGFzcz0iZGIxIGZ3Ij4wMSAwMiAwNyAwOCAwOTwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQiPjE8L3RkPjx0ZCBjbGFzcz0iYjA1IGJhbGxfcmVkIj4yPC90ZD48dGQgY2xhc3M9ImIwNSI%2BMjwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjE8L3RkPjx0ZCBjbGFzcz0iYjA1Ij4zPC90ZD48dGQgY2xhc3M9ImIwNSI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQiPjc8L3RkPjx0ZCBjbGFzcz0iYjA1IGJhbGxfcmVkIj44PC90ZD48dGQgY2xhc3M9ImIwNSBiYWxsX3JlZCI%2BOTwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjI8L3RkPjx0ZCBjbGFzcz0iYjA1IGRiMSI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjIwNTg8L3RkPjx0ZCBjbGFzcz0iYjAyIj4xPC90ZD48dGQgY2xhc3M9ImIwMiI%2BMzwvdGQ%2BPHRkIGNsYXNzPSJiMDIgcmVjdF9ibHVlIj4zPC90ZD48dGQgY2xhc3M9ImIwMiI%2BODwvdGQ%2BPHRkIGNsYXNzPSJiMDIgZGIxIj4xNTwvdGQ%2BPHRkIGNsYXNzPSJiMDEiPjMxNzwvdGQ%2BPHRkIGNsYXNzPSJiMDEiPjI3PC90ZD48dGQgY2xhc3M9ImIwMSI%2BMzwvdGQ%2BPHRkIGNsYXNzPSJiMDEgcmVjdF95ZWxsb3ciPjM8L3RkPjx0ZCBjbGFzcz0iYjAxIj4yPC90ZD48dGQgY2xhc3M9ImIwMSBkYjEiPjMxPC90ZD48dGQgY2xhc3M9ImIwMiI%2BMTE0PC90ZD48dGQgY2xhc3M9ImIwMiI%2BMjk8L3RkPjx0ZCBjbGFzcz0iYjAyIj4xPC90ZD48dGQgY2xhc3M9ImIwMiByZWN0X3JlZCI%2BMzwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjE1PC90ZD48dGQgY2xhc3M9ImIwMiI%2BNTI8L3RkPjwvdGFibGU%2BDQpkZPtGCk0%2B0mcn0sGOHEkYuvybwud5&__VIEWSTATEGENERATOR=578D3B3C&__EVENTVALIDATION=%2FwEWCAKBxNuBAgKQ%2FOqYBwKAk8D2CwKfk8D2CwKek8D2CwKdk8D2CwKck8D2CwKbk8D2CzXqzsrQF3tk2yuuWiA97yk04MOu&drop1=1";
            return GetOpenListFromMainUrl(todaypostdata);
        }
        /// <summary>
        /// 通过主站点抓取昨日开奖列表
        /// </summary>
        /// <returns></returns>
        private Dictionary<string, string> GetYesterdayOpenListFromMainUrl()
        {
            var yesterdaypostdata = "__EVENTTARGET=drop1&__EVENTARGUMENT=&__LASTFOCUS=&__VIEWSTATE=%2FwEPDwULLTEwNzQzMzg2NzYPZBYCAgMPZBYEAgEPEGRkFgFmZAIDDxYCHgRUZXh0BbTSATx0YWJsZSB3aWR0aD0iMTI1MiIgY2xhc3M9InRiayI%2BDQo8dHI%2BPHRoIHdpZHRoPSI3NSIgcm93c3Bhbj0iMiI%2B5pel5pyfPC90aD48dGggd2lkdGg9IjcwIiByb3dzcGFuPSIyIj7mnJ%2Flj7c8L3RoPjx0aCB3aWR0aD0iMTAwIiByb3dzcGFuPSIyIiBjbGFzcz0iZGIxIj7lvIDlpZblj7fnoIE8L3RoPjx0aCB3aWR0aD0iMTAwIiByb3dzcGFuPSIyIiBjbGFzcz0iZGIxIj7nu4TpgIk8L3RoPjx0aCBjb2xzcGFuPSIxMSIgY2xhc3M9ImRiMSI%2B5byA5aWW5Y%2B356CB5YiG5biDPC90aD48dGggY29sc3Bhbj0iNiIgY2xhc3M9ImRiMSI%2B5aWH5pWw5Liq5pWw6LWw5Yq%2FPC90aD48dGggY29sc3Bhbj0iNiIgY2xhc3M9ImRiMSI%2B5aSn5pWw5Liq5pWw6LWw5Yq%2FPC90aD48dGggY29sc3Bhbj0iNiI%2B6LSo5pWw5Liq5pWw6LWw5Yq%2FPC90aD48L3RyPg0KICA8dHIgY2xhc3M9ImRiMiBiZ3RoIj48dGg%2BMDE8L3RoPjx0aD4wMjwvdGg%2BPHRoPjAzPC90aD48dGg%2BMDQ8L3RoPjx0aD4wNTwvdGg%2BPHRoPjA2PC90aD48dGg%2BMDc8L3RoPjx0aD4wODwvdGg%2BPHRoPjA5PC90aD48dGg%2BMTA8L3RoPjx0aCBjbGFzcz0iZGIxIj4xMTwvdGg%2BPHRoPjA8L3RoPjx0aD4xPC90aD48dGg%2BMjwvdGg%2BPHRoPjM8L3RoPjx0aD40PC90aD48dGggY2xhc3M9ImRiMSI%2BNTwvdGg%2BPHRoPjA8L3RoPjx0aD4xPC90aD48dGg%2BMjwvdGg%2BPHRoPjM8L3RoPjx0aD40PC90aD48dGggY2xhc3M9ImRiMSI%2BNTwvdGg%2BPHRoPjA8L3RoPjx0aD4xPC90aD48dGg%2BMjwvdGg%2BPHRoPjM8L3RoPjx0aD40PC90aD48dGg%2BNTwvdGg%2BPC90cj4NCjx0cj4NCjx0ZD4yMDE3LTAyLTA4PC90ZD48dGQgY2xhc3M9InQwMiI%2BMTcwMjA4NzI8L3RkPjx0ZCBjbGFzcz0iZGIxIGZ3Ij4wNiAwOCAwNCAxMSAwOTwvdGQ%2BPHRkIGNsYXNzPSJkYjEgZnciPjA0IDA2IDA4IDA5IDExPC90ZD48dGQgY2xhc3M9ImIwNSI%2BMzwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjQ8L3RkPjx0ZCBjbGFzcz0iYjA1Ij4xPC90ZD48dGQgY2xhc3M9ImIwNSBiYWxsX3JlZCI%2BNDwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjM8L3RkPjx0ZCBjbGFzcz0iYjA1IGJhbGxfcmVkIj42PC90ZD48dGQgY2xhc3M9ImIwNSI%2BMjwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQiPjg8L3RkPjx0ZCBjbGFzcz0iYjA1IGJhbGxfcmVkIj45PC90ZD48dGQgY2xhc3M9ImIwNSI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQgZGIxIj4xMTwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjIwMjk8L3RkPjx0ZCBjbGFzcz0iYjAyIj4zNDwvdGQ%2BPHRkIGNsYXNzPSJiMDIgcmVjdF9ibHVlIj4yPC90ZD48dGQgY2xhc3M9ImIwMiI%2BMjwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjM8L3RkPjx0ZCBjbGFzcz0iYjAyIGRiMSI%2BMTk8L3RkPjx0ZCBjbGFzcz0iYjAxIj4yODg8L3RkPjx0ZCBjbGFzcz0iYjAxIj4yMjwvdGQ%2BPHRkIGNsYXNzPSJiMDEiPjQ8L3RkPjx0ZCBjbGFzcz0iYjAxIj4xPC90ZD48dGQgY2xhc3M9ImIwMSByZWN0X3llbGxvdyI%2BNDwvdGQ%2BPHRkIGNsYXNzPSJiMDEgZGIxIj4yPC90ZD48dGQgY2xhc3M9ImIwMiI%2BODU8L3RkPjx0ZCBjbGFzcz0iYjAyIHJlY3RfcmVkIj4xPC90ZD48dGQgY2xhc3M9ImIwMiI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjM8L3RkPjx0ZCBjbGFzcz0iYjAyIj45PC90ZD48dGQgY2xhc3M9ImIwMiI%2BMjM8L3RkPjx0cj4NCjx0ZD4yMDE3LTAyLTA4PC90ZD48dGQgY2xhc3M9InQwMiI%2BMTcwMjA4NzM8L3RkPjx0ZCBjbGFzcz0iZGIxIGZ3Ij4xMCAwNyAwNCAwNSAwODwvdGQ%2BPHRkIGNsYXNzPSJkYjEgZnciPjA0IDA1IDA3IDA4IDEwPC90ZD48dGQgY2xhc3M9ImIwNSI%2BNDwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjU8L3RkPjx0ZCBjbGFzcz0iYjA1Ij4yPC90ZD48dGQgY2xhc3M9ImIwNSBiYWxsX3JlZCI%2BNDwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQiPjU8L3RkPjx0ZCBjbGFzcz0iYjA1Ij4xPC90ZD48dGQgY2xhc3M9ImIwNSBiYWxsX3JlZCI%2BNzwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQiPjg8L3RkPjx0ZCBjbGFzcz0iYjA1Ij4xPC90ZD48dGQgY2xhc3M9ImIwNSBiYWxsX3JlZCI%2BMTA8L3RkPjx0ZCBjbGFzcz0iYjA1IGRiMSI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjIwMzA8L3RkPjx0ZCBjbGFzcz0iYjAyIj4zNTwvdGQ%2BPHRkIGNsYXNzPSJiMDIgcmVjdF9ibHVlIj4yPC90ZD48dGQgY2xhc3M9ImIwMiI%2BMzwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjQ8L3RkPjx0ZCBjbGFzcz0iYjAyIGRiMSI%2BMjA8L3RkPjx0ZCBjbGFzcz0iYjAxIj4yODk8L3RkPjx0ZCBjbGFzcz0iYjAxIj4yMzwvdGQ%2BPHRkIGNsYXNzPSJiMDEiPjU8L3RkPjx0ZCBjbGFzcz0iYjAxIHJlY3RfeWVsbG93Ij4zPC90ZD48dGQgY2xhc3M9ImIwMSI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDEgZGIxIj4zPC90ZD48dGQgY2xhc3M9ImIwMiI%2BODY8L3RkPjx0ZCBjbGFzcz0iYjAyIj4xPC90ZD48dGQgY2xhc3M9ImIwMiByZWN0X3JlZCI%2BMjwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjQ8L3RkPjx0ZCBjbGFzcz0iYjAyIj4xMDwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjI0PC90ZD48dHI%2BDQo8dGQ%2BMjAxNy0wMi0wODwvdGQ%2BPHRkIGNsYXNzPSJ0MDIiPjE3MDIwODc0PC90ZD48dGQgY2xhc3M9ImRiMSBmdyI%2BMDEgMDIgMDUgMDYgMDQ8L3RkPjx0ZCBjbGFzcz0iZGIxIGZ3Ij4wMSAwMiAwNCAwNSAwNjwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQiPjE8L3RkPjx0ZCBjbGFzcz0iYjA1IGJhbGxfcmVkIj4yPC90ZD48dGQgY2xhc3M9ImIwNSI%2BMzwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQiPjQ8L3RkPjx0ZCBjbGFzcz0iYjA1IGJhbGxfcmVkIj41PC90ZD48dGQgY2xhc3M9ImIwNSBiYWxsX3JlZCI%2BNjwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjE8L3RkPjx0ZCBjbGFzcz0iYjA1Ij4xPC90ZD48dGQgY2xhc3M9ImIwNSI%2BMjwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjE8L3RkPjx0ZCBjbGFzcz0iYjA1IGRiMSI%2BMjwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjIwMzE8L3RkPjx0ZCBjbGFzcz0iYjAyIj4zNjwvdGQ%2BPHRkIGNsYXNzPSJiMDIgcmVjdF9ibHVlIj4yPC90ZD48dGQgY2xhc3M9ImIwMiI%2BNDwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjU8L3RkPjx0ZCBjbGFzcz0iYjAyIGRiMSI%2BMjE8L3RkPjx0ZCBjbGFzcz0iYjAxIj4yOTA8L3RkPjx0ZCBjbGFzcz0iYjAxIHJlY3RfeWVsbG93Ij4xPC90ZD48dGQgY2xhc3M9ImIwMSI%2BNjwvdGQ%2BPHRkIGNsYXNzPSJiMDEiPjE8L3RkPjx0ZCBjbGFzcz0iYjAxIj4yPC90ZD48dGQgY2xhc3M9ImIwMSBkYjEiPjQ8L3RkPjx0ZCBjbGFzcz0iYjAyIj44NzwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjI8L3RkPjx0ZCBjbGFzcz0iYjAyIj4xPC90ZD48dGQgY2xhc3M9ImIwMiByZWN0X3JlZCI%2BMzwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjExPC90ZD48dGQgY2xhc3M9ImIwMiI%2BMjU8L3RkPjx0cj4NCjx0ZD4yMDE3LTAyLTA4PC90ZD48dGQgY2xhc3M9InQwMiI%2BMTcwMjA4NzU8L3RkPjx0ZCBjbGFzcz0iZGIxIGZ3Ij4wNSAwMSAwOSAxMSAwNDwvdGQ%2BPHRkIGNsYXNzPSJkYjEgZnciPjAxIDA0IDA1IDA5IDExPC90ZD48dGQgY2xhc3M9ImIwNSBiYWxsX3JlZCI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjE8L3RkPjx0ZCBjbGFzcz0iYjA1Ij40PC90ZD48dGQgY2xhc3M9ImIwNSBiYWxsX3JlZCI%2BNDwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQiPjU8L3RkPjx0ZCBjbGFzcz0iYjA1Ij4xPC90ZD48dGQgY2xhc3M9ImIwNSI%2BMjwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjI8L3RkPjx0ZCBjbGFzcz0iYjA1IGJhbGxfcmVkIj45PC90ZD48dGQgY2xhc3M9ImIwNSI%2BMjwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQgZGIxIj4xMTwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjIwMzI8L3RkPjx0ZCBjbGFzcz0iYjAyIj4zNzwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjE8L3RkPjx0ZCBjbGFzcz0iYjAyIj41PC90ZD48dGQgY2xhc3M9ImIwMiByZWN0X2JsdWUiPjQ8L3RkPjx0ZCBjbGFzcz0iYjAyIGRiMSI%2BMjI8L3RkPjx0ZCBjbGFzcz0iYjAxIj4yOTE8L3RkPjx0ZCBjbGFzcz0iYjAxIj4xPC90ZD48dGQgY2xhc3M9ImIwMSByZWN0X3llbGxvdyI%2BMjwvdGQ%2BPHRkIGNsYXNzPSJiMDEiPjI8L3RkPjx0ZCBjbGFzcz0iYjAxIj4zPC90ZD48dGQgY2xhc3M9ImIwMSBkYjEiPjU8L3RkPjx0ZCBjbGFzcz0iYjAyIj44ODwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjM8L3RkPjx0ZCBjbGFzcz0iYjAyIj4yPC90ZD48dGQgY2xhc3M9ImIwMiByZWN0X3JlZCI%2BMzwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjEyPC90ZD48dGQgY2xhc3M9ImIwMiI%2BMjY8L3RkPjx0ciBjbGFzcz0iZGIyIj4NCjx0ZD4yMDE3LTAyLTA4PC90ZD48dGQgY2xhc3M9InQwMiI%2BMTcwMjA4NzY8L3RkPjx0ZCBjbGFzcz0iZGIxIGZ3Ij4wOSAwMyAwNCAwNyAxMTwvdGQ%2BPHRkIGNsYXNzPSJkYjEgZnciPjAzIDA0IDA3IDA5IDExPC90ZD48dGQgY2xhc3M9ImIwNSI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjI8L3RkPjx0ZCBjbGFzcz0iYjA1IGJhbGxfcmVkIj4zPC90ZD48dGQgY2xhc3M9ImIwNSBiYWxsX3JlZCI%2BNDwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjE8L3RkPjx0ZCBjbGFzcz0iYjA1Ij4yPC90ZD48dGQgY2xhc3M9ImIwNSBiYWxsX3JlZCI%2BNzwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjM8L3RkPjx0ZCBjbGFzcz0iYjA1IGJhbGxfcmVkIj45PC90ZD48dGQgY2xhc3M9ImIwNSI%2BMzwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQgZGIxIj4xMTwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjIwMzM8L3RkPjx0ZCBjbGFzcz0iYjAyIj4zODwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjI8L3RkPjx0ZCBjbGFzcz0iYjAyIj42PC90ZD48dGQgY2xhc3M9ImIwMiByZWN0X2JsdWUiPjQ8L3RkPjx0ZCBjbGFzcz0iYjAyIGRiMSI%2BMjM8L3RkPjx0ZCBjbGFzcz0iYjAxIj4yOTI8L3RkPjx0ZCBjbGFzcz0iYjAxIj4yPC90ZD48dGQgY2xhc3M9ImIwMSI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDEgcmVjdF95ZWxsb3ciPjM8L3RkPjx0ZCBjbGFzcz0iYjAxIj40PC90ZD48dGQgY2xhc3M9ImIwMSBkYjEiPjY8L3RkPjx0ZCBjbGFzcz0iYjAyIj44OTwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjQ8L3RkPjx0ZCBjbGFzcz0iYjAyIj4zPC90ZD48dGQgY2xhc3M9ImIwMiByZWN0X3JlZCI%2BMzwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjEzPC90ZD48dGQgY2xhc3M9ImIwMiI%2BMjc8L3RkPjx0cj4NCjx0ZD4yMDE3LTAyLTA4PC90ZD48dGQgY2xhc3M9InQwMiI%2BMTcwMjA4Nzc8L3RkPjx0ZCBjbGFzcz0iZGIxIGZ3Ij4wNSAwOCAwMiAxMSAwNDwvdGQ%2BPHRkIGNsYXNzPSJkYjEgZnciPjAyIDA0IDA1IDA4IDExPC90ZD48dGQgY2xhc3M9ImIwNSI%2BMjwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQiPjI8L3RkPjx0ZCBjbGFzcz0iYjA1Ij4xPC90ZD48dGQgY2xhc3M9ImIwNSBiYWxsX3JlZCI%2BNDwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQiPjU8L3RkPjx0ZCBjbGFzcz0iYjA1Ij4zPC90ZD48dGQgY2xhc3M9ImIwNSI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQiPjg8L3RkPjx0ZCBjbGFzcz0iYjA1Ij4xPC90ZD48dGQgY2xhc3M9ImIwNSI%2BNDwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQgZGIxIj4xMTwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjIwMzQ8L3RkPjx0ZCBjbGFzcz0iYjAyIj4zOTwvdGQ%2BPHRkIGNsYXNzPSJiMDIgcmVjdF9ibHVlIj4yPC90ZD48dGQgY2xhc3M9ImIwMiI%2BNzwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjE8L3RkPjx0ZCBjbGFzcz0iYjAyIGRiMSI%2BMjQ8L3RkPjx0ZCBjbGFzcz0iYjAxIj4yOTM8L3RkPjx0ZCBjbGFzcz0iYjAxIj4zPC90ZD48dGQgY2xhc3M9ImIwMSByZWN0X3llbGxvdyI%2BMjwvdGQ%2BPHRkIGNsYXNzPSJiMDEiPjE8L3RkPjx0ZCBjbGFzcz0iYjAxIj41PC90ZD48dGQgY2xhc3M9ImIwMSBkYjEiPjc8L3RkPjx0ZCBjbGFzcz0iYjAyIj45MDwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjU8L3RkPjx0ZCBjbGFzcz0iYjAyIj40PC90ZD48dGQgY2xhc3M9ImIwMiByZWN0X3JlZCI%2BMzwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjE0PC90ZD48dGQgY2xhc3M9ImIwMiI%2BMjg8L3RkPjx0cj4NCjx0ZD4yMDE3LTAyLTA4PC90ZD48dGQgY2xhc3M9InQwMiI%2BMTcwMjA4Nzg8L3RkPjx0ZCBjbGFzcz0iZGIxIGZ3Ij4wMSAwMyAwNCAwOCAwNjwvdGQ%2BPHRkIGNsYXNzPSJkYjEgZnciPjAxIDAzIDA0IDA2IDA4PC90ZD48dGQgY2xhc3M9ImIwNSBiYWxsX3JlZCI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjE8L3RkPjx0ZCBjbGFzcz0iYjA1IGJhbGxfcmVkIj4zPC90ZD48dGQgY2xhc3M9ImIwNSBiYWxsX3JlZCI%2BNDwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjE8L3RkPjx0ZCBjbGFzcz0iYjA1IGJhbGxfcmVkIj42PC90ZD48dGQgY2xhc3M9ImIwNSI%2BMjwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQiPjg8L3RkPjx0ZCBjbGFzcz0iYjA1Ij4yPC90ZD48dGQgY2xhc3M9ImIwNSI%2BNTwvdGQ%2BPHRkIGNsYXNzPSJiMDUgZGIxIj4xPC90ZD48dGQgY2xhc3M9ImIwMiI%2BMjAzNTwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjQwPC90ZD48dGQgY2xhc3M9ImIwMiByZWN0X2JsdWUiPjI8L3RkPjx0ZCBjbGFzcz0iYjAyIj44PC90ZD48dGQgY2xhc3M9ImIwMiI%2BMjwvdGQ%2BPHRkIGNsYXNzPSJiMDIgZGIxIj4yNTwvdGQ%2BPHRkIGNsYXNzPSJiMDEiPjI5NDwvdGQ%2BPHRkIGNsYXNzPSJiMDEiPjQ8L3RkPjx0ZCBjbGFzcz0iYjAxIHJlY3RfeWVsbG93Ij4yPC90ZD48dGQgY2xhc3M9ImIwMSI%2BMjwvdGQ%2BPHRkIGNsYXNzPSJiMDEiPjY8L3RkPjx0ZCBjbGFzcz0iYjAxIGRiMSI%2BODwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjkxPC90ZD48dGQgY2xhc3M9ImIwMiI%2BNjwvdGQ%2BPHRkIGNsYXNzPSJiMDIgcmVjdF9yZWQiPjI8L3RkPjx0ZCBjbGFzcz0iYjAyIj4xPC90ZD48dGQgY2xhc3M9ImIwMiI%2BMTU8L3RkPjx0ZCBjbGFzcz0iYjAyIj4yOTwvdGQ%2BPHRyPg0KPHRkPjIwMTctMDItMDg8L3RkPjx0ZCBjbGFzcz0idDAyIj4xNzAyMDg3OTwvdGQ%2BPHRkIGNsYXNzPSJkYjEgZnciPjAyIDA1IDA2IDEwIDA0PC90ZD48dGQgY2xhc3M9ImRiMSBmdyI%2BMDIgMDQgMDUgMDYgMTA8L3RkPjx0ZCBjbGFzcz0iYjA1Ij4xPC90ZD48dGQgY2xhc3M9ImIwNSBiYWxsX3JlZCI%2BMjwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjE8L3RkPjx0ZCBjbGFzcz0iYjA1IGJhbGxfcmVkIj40PC90ZD48dGQgY2xhc3M9ImIwNSBiYWxsX3JlZCI%2BNTwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQiPjY8L3RkPjx0ZCBjbGFzcz0iYjA1Ij4zPC90ZD48dGQgY2xhc3M9ImIwNSI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjM8L3RkPjx0ZCBjbGFzcz0iYjA1IGJhbGxfcmVkIj4xMDwvdGQ%2BPHRkIGNsYXNzPSJiMDUgZGIxIj4yPC90ZD48dGQgY2xhc3M9ImIwMiI%2BMjAzNjwvdGQ%2BPHRkIGNsYXNzPSJiMDIgcmVjdF9ibHVlIj4xPC90ZD48dGQgY2xhc3M9ImIwMiI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjk8L3RkPjx0ZCBjbGFzcz0iYjAyIj4zPC90ZD48dGQgY2xhc3M9ImIwMiBkYjEiPjI2PC90ZD48dGQgY2xhc3M9ImIwMSI%2BMjk1PC90ZD48dGQgY2xhc3M9ImIwMSI%2BNTwvdGQ%2BPHRkIGNsYXNzPSJiMDEgcmVjdF95ZWxsb3ciPjI8L3RkPjx0ZCBjbGFzcz0iYjAxIj4zPC90ZD48dGQgY2xhc3M9ImIwMSI%2BNzwvdGQ%2BPHRkIGNsYXNzPSJiMDEgZGIxIj45PC90ZD48dGQgY2xhc3M9ImIwMiI%2BOTI8L3RkPjx0ZCBjbGFzcz0iYjAyIj43PC90ZD48dGQgY2xhc3M9ImIwMiByZWN0X3JlZCI%2BMjwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjI8L3RkPjx0ZCBjbGFzcz0iYjAyIj4xNjwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjMwPC90ZD48dHI%2BDQo8dGQ%2BMjAxNy0wMi0wODwvdGQ%2BPHRkIGNsYXNzPSJ0MDIiPjE3MDIwODgwPC90ZD48dGQgY2xhc3M9ImRiMSBmdyI%2BMDQgMDEgMDcgMDUgMTE8L3RkPjx0ZCBjbGFzcz0iZGIxIGZ3Ij4wMSAwNCAwNSAwNyAxMTwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQiPjE8L3RkPjx0ZCBjbGFzcz0iYjA1Ij4xPC90ZD48dGQgY2xhc3M9ImIwNSI%2BMjwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQiPjQ8L3RkPjx0ZCBjbGFzcz0iYjA1IGJhbGxfcmVkIj41PC90ZD48dGQgY2xhc3M9ImIwNSI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQiPjc8L3RkPjx0ZCBjbGFzcz0iYjA1Ij4yPC90ZD48dGQgY2xhc3M9ImIwNSI%2BNDwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjE8L3RkPjx0ZCBjbGFzcz0iYjA1IGJhbGxfcmVkIGRiMSI%2BMTE8L3RkPjx0ZCBjbGFzcz0iYjAyIj4yMDM3PC90ZD48dGQgY2xhc3M9ImIwMiI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjI8L3RkPjx0ZCBjbGFzcz0iYjAyIj4xMDwvdGQ%2BPHRkIGNsYXNzPSJiMDIgcmVjdF9ibHVlIj40PC90ZD48dGQgY2xhc3M9ImIwMiBkYjEiPjI3PC90ZD48dGQgY2xhc3M9ImIwMSI%2BMjk2PC90ZD48dGQgY2xhc3M9ImIwMSI%2BNjwvdGQ%2BPHRkIGNsYXNzPSJiMDEgcmVjdF95ZWxsb3ciPjI8L3RkPjx0ZCBjbGFzcz0iYjAxIj40PC90ZD48dGQgY2xhc3M9ImIwMSI%2BODwvdGQ%2BPHRkIGNsYXNzPSJiMDEgZGIxIj4xMDwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjkzPC90ZD48dGQgY2xhc3M9ImIwMiI%2BODwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjE8L3RkPjx0ZCBjbGFzcz0iYjAyIj4zPC90ZD48dGQgY2xhc3M9ImIwMiByZWN0X3JlZCI%2BNDwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjMxPC90ZD48dHIgY2xhc3M9ImRiMiI%2BDQo8dGQ%2BMjAxNy0wMi0wODwvdGQ%2BPHRkIGNsYXNzPSJ0MDIiPjE3MDIwODgxPC90ZD48dGQgY2xhc3M9ImRiMSBmdyI%2BMDggMTEgMDIgMDcgMTA8L3RkPjx0ZCBjbGFzcz0iZGIxIGZ3Ij4wMiAwNyAwOCAxMCAxMTwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjE8L3RkPjx0ZCBjbGFzcz0iYjA1IGJhbGxfcmVkIj4yPC90ZD48dGQgY2xhc3M9ImIwNSI%2BMzwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjE8L3RkPjx0ZCBjbGFzcz0iYjA1Ij4xPC90ZD48dGQgY2xhc3M9ImIwNSI%2BMjwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQiPjc8L3RkPjx0ZCBjbGFzcz0iYjA1IGJhbGxfcmVkIj44PC90ZD48dGQgY2xhc3M9ImIwNSI%2BNTwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQiPjEwPC90ZD48dGQgY2xhc3M9ImIwNSBiYWxsX3JlZCBkYjEiPjExPC90ZD48dGQgY2xhc3M9ImIwMiI%2BMjAzODwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjI8L3RkPjx0ZCBjbGFzcz0iYjAyIHJlY3RfYmx1ZSI%2BMjwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjExPC90ZD48dGQgY2xhc3M9ImIwMiI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDIgZGIxIj4yODwvdGQ%2BPHRkIGNsYXNzPSJiMDEiPjI5NzwvdGQ%2BPHRkIGNsYXNzPSJiMDEiPjc8L3RkPjx0ZCBjbGFzcz0iYjAxIj4xPC90ZD48dGQgY2xhc3M9ImIwMSI%2BNTwvdGQ%2BPHRkIGNsYXNzPSJiMDEgcmVjdF95ZWxsb3ciPjQ8L3RkPjx0ZCBjbGFzcz0iYjAxIGRiMSI%2BMTE8L3RkPjx0ZCBjbGFzcz0iYjAyIj45NDwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjk8L3RkPjx0ZCBjbGFzcz0iYjAyIj4yPC90ZD48dGQgY2xhc3M9ImIwMiByZWN0X3JlZCI%2BMzwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjE8L3RkPjx0ZCBjbGFzcz0iYjAyIj4zMjwvdGQ%2BPHRyPg0KPHRkPjIwMTctMDItMDg8L3RkPjx0ZCBjbGFzcz0idDAyIj4xNzAyMDg4MjwvdGQ%2BPHRkIGNsYXNzPSJkYjEgZnciPjA2IDExIDA4IDA0IDA3PC90ZD48dGQgY2xhc3M9ImRiMSBmdyI%2BMDQgMDYgMDcgMDggMTE8L3RkPjx0ZCBjbGFzcz0iYjA1Ij4yPC90ZD48dGQgY2xhc3M9ImIwNSI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjQ8L3RkPjx0ZCBjbGFzcz0iYjA1IGJhbGxfcmVkIj40PC90ZD48dGQgY2xhc3M9ImIwNSI%2BMjwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQiPjY8L3RkPjx0ZCBjbGFzcz0iYjA1IGJhbGxfcmVkIj43PC90ZD48dGQgY2xhc3M9ImIwNSBiYWxsX3JlZCI%2BODwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjY8L3RkPjx0ZCBjbGFzcz0iYjA1Ij4xPC90ZD48dGQgY2xhc3M9ImIwNSBiYWxsX3JlZCBkYjEiPjExPC90ZD48dGQgY2xhc3M9ImIwMiI%2BMjAzOTwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjM8L3RkPjx0ZCBjbGFzcz0iYjAyIHJlY3RfYmx1ZSI%2BMjwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjEyPC90ZD48dGQgY2xhc3M9ImIwMiI%2BMjwvdGQ%2BPHRkIGNsYXNzPSJiMDIgZGIxIj4yOTwvdGQ%2BPHRkIGNsYXNzPSJiMDEiPjI5ODwvdGQ%2BPHRkIGNsYXNzPSJiMDEiPjg8L3RkPjx0ZCBjbGFzcz0iYjAxIj4yPC90ZD48dGQgY2xhc3M9ImIwMSI%2BNjwvdGQ%2BPHRkIGNsYXNzPSJiMDEgcmVjdF95ZWxsb3ciPjQ8L3RkPjx0ZCBjbGFzcz0iYjAxIGRiMSI%2BMTI8L3RkPjx0ZCBjbGFzcz0iYjAyIj45NTwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjEwPC90ZD48dGQgY2xhc3M9ImIwMiByZWN0X3JlZCI%2BMjwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjE8L3RkPjx0ZCBjbGFzcz0iYjAyIj4yPC90ZD48dGQgY2xhc3M9ImIwMiI%2BMzM8L3RkPjx0cj4NCjx0ZD4yMDE3LTAyLTA4PC90ZD48dGQgY2xhc3M9InQwMiI%2BMTcwMjA4ODM8L3RkPjx0ZCBjbGFzcz0iZGIxIGZ3Ij4wMyAwNiAxMCAwOSAwMjwvdGQ%2BPHRkIGNsYXNzPSJkYjEgZnciPjAyIDAzIDA2IDA5IDEwPC90ZD48dGQgY2xhc3M9ImIwNSI%2BMzwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQiPjI8L3RkPjx0ZCBjbGFzcz0iYjA1IGJhbGxfcmVkIj4zPC90ZD48dGQgY2xhc3M9ImIwNSI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjM8L3RkPjx0ZCBjbGFzcz0iYjA1IGJhbGxfcmVkIj42PC90ZD48dGQgY2xhc3M9ImIwNSI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjE8L3RkPjx0ZCBjbGFzcz0iYjA1IGJhbGxfcmVkIj45PC90ZD48dGQgY2xhc3M9ImIwNSBiYWxsX3JlZCI%2BMTA8L3RkPjx0ZCBjbGFzcz0iYjA1IGRiMSI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjIwNDA8L3RkPjx0ZCBjbGFzcz0iYjAyIj40PC90ZD48dGQgY2xhc3M9ImIwMiByZWN0X2JsdWUiPjI8L3RkPjx0ZCBjbGFzcz0iYjAyIj4xMzwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjM8L3RkPjx0ZCBjbGFzcz0iYjAyIGRiMSI%2BMzA8L3RkPjx0ZCBjbGFzcz0iYjAxIj4yOTk8L3RkPjx0ZCBjbGFzcz0iYjAxIj45PC90ZD48dGQgY2xhc3M9ImIwMSI%2BMzwvdGQ%2BPHRkIGNsYXNzPSJiMDEgcmVjdF95ZWxsb3ciPjM8L3RkPjx0ZCBjbGFzcz0iYjAxIj4xPC90ZD48dGQgY2xhc3M9ImIwMSBkYjEiPjEzPC90ZD48dGQgY2xhc3M9ImIwMiI%2BOTY8L3RkPjx0ZCBjbGFzcz0iYjAyIj4xMTwvdGQ%2BPHRkIGNsYXNzPSJiMDIgcmVjdF9yZWQiPjI8L3RkPjx0ZCBjbGFzcz0iYjAyIj4yPC90ZD48dGQgY2xhc3M9ImIwMiI%2BMzwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjM0PC90ZD48dHI%2BDQo8dGQ%2BMjAxNy0wMi0wODwvdGQ%2BPHRkIGNsYXNzPSJ0MDIiPjE3MDIwODg0PC90ZD48dGQgY2xhc3M9ImRiMSBmdyI%2BMDIgMDEgMDMgMTAgMDc8L3RkPjx0ZCBjbGFzcz0iZGIxIGZ3Ij4wMSAwMiAwMyAwNyAxMDwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQiPjE8L3RkPjx0ZCBjbGFzcz0iYjA1IGJhbGxfcmVkIj4yPC90ZD48dGQgY2xhc3M9ImIwNSBiYWxsX3JlZCI%2BMzwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjI8L3RkPjx0ZCBjbGFzcz0iYjA1Ij40PC90ZD48dGQgY2xhc3M9ImIwNSI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQiPjc8L3RkPjx0ZCBjbGFzcz0iYjA1Ij4yPC90ZD48dGQgY2xhc3M9ImIwNSI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQiPjEwPC90ZD48dGQgY2xhc3M9ImIwNSBkYjEiPjI8L3RkPjx0ZCBjbGFzcz0iYjAyIj4yMDQxPC90ZD48dGQgY2xhc3M9ImIwMiI%2BNTwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjE8L3RkPjx0ZCBjbGFzcz0iYjAyIHJlY3RfYmx1ZSI%2BMzwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjQ8L3RkPjx0ZCBjbGFzcz0iYjAyIGRiMSI%2BMzE8L3RkPjx0ZCBjbGFzcz0iYjAxIj4zMDA8L3RkPjx0ZCBjbGFzcz0iYjAxIj4xMDwvdGQ%2BPHRkIGNsYXNzPSJiMDEgcmVjdF95ZWxsb3ciPjI8L3RkPjx0ZCBjbGFzcz0iYjAxIj4xPC90ZD48dGQgY2xhc3M9ImIwMSI%2BMjwvdGQ%2BPHRkIGNsYXNzPSJiMDEgZGIxIj4xNDwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjk3PC90ZD48dGQgY2xhc3M9ImIwMiI%2BMTI8L3RkPjx0ZCBjbGFzcz0iYjAyIj4xPC90ZD48dGQgY2xhc3M9ImIwMiI%2BMzwvdGQ%2BPHRkIGNsYXNzPSJiMDIgcmVjdF9yZWQiPjQ8L3RkPjx0ZCBjbGFzcz0iYjAyIj4zNTwvdGQ%2BPHRyPg0KPHRkPjIwMTctMDItMDg8L3RkPjx0ZCBjbGFzcz0idDAyIj4xNzAyMDg4NTwvdGQ%2BPHRkIGNsYXNzPSJkYjEgZnciPjAxIDAzIDExIDA1IDEwPC90ZD48dGQgY2xhc3M9ImRiMSBmdyI%2BMDEgMDMgMDUgMTAgMTE8L3RkPjx0ZCBjbGFzcz0iYjA1IGJhbGxfcmVkIj4xPC90ZD48dGQgY2xhc3M9ImIwNSI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQiPjM8L3RkPjx0ZCBjbGFzcz0iYjA1Ij4zPC90ZD48dGQgY2xhc3M9ImIwNSBiYWxsX3JlZCI%2BNTwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjI8L3RkPjx0ZCBjbGFzcz0iYjA1Ij4xPC90ZD48dGQgY2xhc3M9ImIwNSI%2BMzwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjI8L3RkPjx0ZCBjbGFzcz0iYjA1IGJhbGxfcmVkIj4xMDwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQgZGIxIj4xMTwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjIwNDI8L3RkPjx0ZCBjbGFzcz0iYjAyIj42PC90ZD48dGQgY2xhc3M9ImIwMiI%2BMjwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjE8L3RkPjx0ZCBjbGFzcz0iYjAyIHJlY3RfYmx1ZSI%2BNDwvdGQ%2BPHRkIGNsYXNzPSJiMDIgZGIxIj4zMjwvdGQ%2BPHRkIGNsYXNzPSJiMDEiPjMwMTwvdGQ%2BPHRkIGNsYXNzPSJiMDEiPjExPC90ZD48dGQgY2xhc3M9ImIwMSByZWN0X3llbGxvdyI%2BMjwvdGQ%2BPHRkIGNsYXNzPSJiMDEiPjI8L3RkPjx0ZCBjbGFzcz0iYjAxIj4zPC90ZD48dGQgY2xhc3M9ImIwMSBkYjEiPjE1PC90ZD48dGQgY2xhc3M9ImIwMiI%2BOTg8L3RkPjx0ZCBjbGFzcz0iYjAyIj4xMzwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjI8L3RkPjx0ZCBjbGFzcz0iYjAyIj40PC90ZD48dGQgY2xhc3M9ImIwMiByZWN0X3JlZCI%2BNDwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjM2PC90ZD48dHIgY2xhc3M9ImRiMiI%2BDQo8dGQ%2BMjAxNy0wMi0wOTwvdGQ%2BPHRkIGNsYXNzPSJ0MDIiPjE3MDIwOTAxPC90ZD48dGQgY2xhc3M9ImRiMSBmdyI%2BMDcgMDUgMTEgMDMgMDk8L3RkPjx0ZCBjbGFzcz0iZGIxIGZ3Ij4wMyAwNSAwNyAwOSAxMTwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjE8L3RkPjx0ZCBjbGFzcz0iYjA1Ij4yPC90ZD48dGQgY2xhc3M9ImIwNSBiYWxsX3JlZCI%2BMzwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjQ8L3RkPjx0ZCBjbGFzcz0iYjA1IGJhbGxfcmVkIj41PC90ZD48dGQgY2xhc3M9ImIwNSI%2BMzwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQiPjc8L3RkPjx0ZCBjbGFzcz0iYjA1Ij40PC90ZD48dGQgY2xhc3M9ImIwNSBiYWxsX3JlZCI%2BOTwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjE8L3RkPjx0ZCBjbGFzcz0iYjA1IGJhbGxfcmVkIGRiMSI%2BMTE8L3RkPjx0ZCBjbGFzcz0iYjAyIj4yMDQzPC90ZD48dGQgY2xhc3M9ImIwMiI%2BNzwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjM8L3RkPjx0ZCBjbGFzcz0iYjAyIj4yPC90ZD48dGQgY2xhc3M9ImIwMiI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDIgcmVjdF9ibHVlIGRiMSI%2BNTwvdGQ%2BPHRkIGNsYXNzPSJiMDEiPjMwMjwvdGQ%2BPHRkIGNsYXNzPSJiMDEiPjEyPC90ZD48dGQgY2xhc3M9ImIwMSI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDEgcmVjdF95ZWxsb3ciPjM8L3RkPjx0ZCBjbGFzcz0iYjAxIj40PC90ZD48dGQgY2xhc3M9ImIwMSBkYjEiPjE2PC90ZD48dGQgY2xhc3M9ImIwMiI%2BOTk8L3RkPjx0ZCBjbGFzcz0iYjAyIj4xNDwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjM8L3RkPjx0ZCBjbGFzcz0iYjAyIj41PC90ZD48dGQgY2xhc3M9ImIwMiByZWN0X3JlZCI%2BNDwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjM3PC90ZD48dHI%2BDQo8dGQ%2BMjAxNy0wMi0wOTwvdGQ%2BPHRkIGNsYXNzPSJ0MDIiPjE3MDIwOTAyPC90ZD48dGQgY2xhc3M9ImRiMSBmdyI%2BMDEgMTAgMDcgMDQgMDI8L3RkPjx0ZCBjbGFzcz0iZGIxIGZ3Ij4wMSAwMiAwNCAwNyAxMDwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQiPjE8L3RkPjx0ZCBjbGFzcz0iYjA1IGJhbGxfcmVkIj4yPC90ZD48dGQgY2xhc3M9ImIwNSI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQiPjQ8L3RkPjx0ZCBjbGFzcz0iYjA1Ij4xPC90ZD48dGQgY2xhc3M9ImIwNSI%2BNDwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQiPjc8L3RkPjx0ZCBjbGFzcz0iYjA1Ij41PC90ZD48dGQgY2xhc3M9ImIwNSI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQiPjEwPC90ZD48dGQgY2xhc3M9ImIwNSBkYjEiPjE8L3RkPjx0ZCBjbGFzcz0iYjAyIj4yMDQ0PC90ZD48dGQgY2xhc3M9ImIwMiI%2BODwvdGQ%2BPHRkIGNsYXNzPSJiMDIgcmVjdF9ibHVlIj4yPC90ZD48dGQgY2xhc3M9ImIwMiI%2BMzwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjI8L3RkPjx0ZCBjbGFzcz0iYjAyIGRiMSI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDEiPjMwMzwvdGQ%2BPHRkIGNsYXNzPSJiMDEiPjEzPC90ZD48dGQgY2xhc3M9ImIwMSByZWN0X3llbGxvdyI%2BMjwvdGQ%2BPHRkIGNsYXNzPSJiMDEiPjE8L3RkPjx0ZCBjbGFzcz0iYjAxIj41PC90ZD48dGQgY2xhc3M9ImIwMSBkYjEiPjE3PC90ZD48dGQgY2xhc3M9ImIwMiI%2BMTAwPC90ZD48dGQgY2xhc3M9ImIwMiI%2BMTU8L3RkPjx0ZCBjbGFzcz0iYjAyIj40PC90ZD48dGQgY2xhc3M9ImIwMiByZWN0X3JlZCI%2BMzwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjE8L3RkPjx0ZCBjbGFzcz0iYjAyIj4zODwvdGQ%2BPHRyPg0KPHRkPjIwMTctMDItMDk8L3RkPjx0ZCBjbGFzcz0idDAyIj4xNzAyMDkwMzwvdGQ%2BPHRkIGNsYXNzPSJkYjEgZnciPjAyIDAxIDA2IDExIDEwPC90ZD48dGQgY2xhc3M9ImRiMSBmdyI%2BMDEgMDIgMDYgMTAgMTE8L3RkPjx0ZCBjbGFzcz0iYjA1IGJhbGxfcmVkIj4xPC90ZD48dGQgY2xhc3M9ImIwNSBiYWxsX3JlZCI%2BMjwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjI8L3RkPjx0ZCBjbGFzcz0iYjA1Ij4xPC90ZD48dGQgY2xhc3M9ImIwNSI%2BMjwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQiPjY8L3RkPjx0ZCBjbGFzcz0iYjA1Ij4xPC90ZD48dGQgY2xhc3M9ImIwNSI%2BNjwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjI8L3RkPjx0ZCBjbGFzcz0iYjA1IGJhbGxfcmVkIj4xMDwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQgZGIxIj4xMTwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjIwNDU8L3RkPjx0ZCBjbGFzcz0iYjAyIj45PC90ZD48dGQgY2xhc3M9ImIwMiByZWN0X2JsdWUiPjI8L3RkPjx0ZCBjbGFzcz0iYjAyIj40PC90ZD48dGQgY2xhc3M9ImIwMiI%2BMzwvdGQ%2BPHRkIGNsYXNzPSJiMDIgZGIxIj4yPC90ZD48dGQgY2xhc3M9ImIwMSI%2BMzA0PC90ZD48dGQgY2xhc3M9ImIwMSI%2BMTQ8L3RkPjx0ZCBjbGFzcz0iYjAxIj4xPC90ZD48dGQgY2xhc3M9ImIwMSByZWN0X3llbGxvdyI%2BMzwvdGQ%2BPHRkIGNsYXNzPSJiMDEiPjY8L3RkPjx0ZCBjbGFzcz0iYjAxIGRiMSI%2BMTg8L3RkPjx0ZCBjbGFzcz0iYjAyIj4xMDE8L3RkPjx0ZCBjbGFzcz0iYjAyIj4xNjwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjU8L3RkPjx0ZCBjbGFzcz0iYjAyIHJlY3RfcmVkIj4zPC90ZD48dGQgY2xhc3M9ImIwMiI%2BMjwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjM5PC90ZD48dHI%2BDQo8dGQ%2BMjAxNy0wMi0wOTwvdGQ%2BPHRkIGNsYXNzPSJ0MDIiPjE3MDIwOTA0PC90ZD48dGQgY2xhc3M9ImRiMSBmdyI%2BMDggMDkgMDEgMTEgMDI8L3RkPjx0ZCBjbGFzcz0iZGIxIGZ3Ij4wMSAwMiAwOCAwOSAxMTwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQiPjE8L3RkPjx0ZCBjbGFzcz0iYjA1IGJhbGxfcmVkIj4yPC90ZD48dGQgY2xhc3M9ImIwNSI%2BMzwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjI8L3RkPjx0ZCBjbGFzcz0iYjA1Ij4zPC90ZD48dGQgY2xhc3M9ImIwNSI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjI8L3RkPjx0ZCBjbGFzcz0iYjA1IGJhbGxfcmVkIj44PC90ZD48dGQgY2xhc3M9ImIwNSBiYWxsX3JlZCI%2BOTwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjE8L3RkPjx0ZCBjbGFzcz0iYjA1IGJhbGxfcmVkIGRiMSI%2BMTE8L3RkPjx0ZCBjbGFzcz0iYjAyIj4yMDQ2PC90ZD48dGQgY2xhc3M9ImIwMiI%2BMTA8L3RkPjx0ZCBjbGFzcz0iYjAyIj4xPC90ZD48dGQgY2xhc3M9ImIwMiByZWN0X2JsdWUiPjM8L3RkPjx0ZCBjbGFzcz0iYjAyIj40PC90ZD48dGQgY2xhc3M9ImIwMiBkYjEiPjM8L3RkPjx0ZCBjbGFzcz0iYjAxIj4zMDU8L3RkPjx0ZCBjbGFzcz0iYjAxIj4xNTwvdGQ%2BPHRkIGNsYXNzPSJiMDEiPjI8L3RkPjx0ZCBjbGFzcz0iYjAxIHJlY3RfeWVsbG93Ij4zPC90ZD48dGQgY2xhc3M9ImIwMSI%2BNzwvdGQ%2BPHRkIGNsYXNzPSJiMDEgZGIxIj4xOTwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjEwMjwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjE3PC90ZD48dGQgY2xhc3M9ImIwMiI%2BNjwvdGQ%2BPHRkIGNsYXNzPSJiMDIgcmVjdF9yZWQiPjM8L3RkPjx0ZCBjbGFzcz0iYjAyIj4zPC90ZD48dGQgY2xhc3M9ImIwMiI%2BNDA8L3RkPjx0cj4NCjx0ZD4yMDE3LTAyLTA5PC90ZD48dGQgY2xhc3M9InQwMiI%2BMTcwMjA5MDU8L3RkPjx0ZCBjbGFzcz0iZGIxIGZ3Ij4wMiAxMCAwNSAwNiAwOTwvdGQ%2BPHRkIGNsYXNzPSJkYjEgZnciPjAyIDA1IDA2IDA5IDEwPC90ZD48dGQgY2xhc3M9ImIwNSI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQiPjI8L3RkPjx0ZCBjbGFzcz0iYjA1Ij40PC90ZD48dGQgY2xhc3M9ImIwNSI%2BMzwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQiPjU8L3RkPjx0ZCBjbGFzcz0iYjA1IGJhbGxfcmVkIj42PC90ZD48dGQgY2xhc3M9ImIwNSI%2BMzwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjE8L3RkPjx0ZCBjbGFzcz0iYjA1IGJhbGxfcmVkIj45PC90ZD48dGQgY2xhc3M9ImIwNSBiYWxsX3JlZCI%2BMTA8L3RkPjx0ZCBjbGFzcz0iYjA1IGRiMSI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjIwNDc8L3RkPjx0ZCBjbGFzcz0iYjAyIj4xMTwvdGQ%2BPHRkIGNsYXNzPSJiMDIgcmVjdF9ibHVlIj4yPC90ZD48dGQgY2xhc3M9ImIwMiI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjU8L3RkPjx0ZCBjbGFzcz0iYjAyIGRiMSI%2BNDwvdGQ%2BPHRkIGNsYXNzPSJiMDEiPjMwNjwvdGQ%2BPHRkIGNsYXNzPSJiMDEiPjE2PC90ZD48dGQgY2xhc3M9ImIwMSI%2BMzwvdGQ%2BPHRkIGNsYXNzPSJiMDEgcmVjdF95ZWxsb3ciPjM8L3RkPjx0ZCBjbGFzcz0iYjAxIj44PC90ZD48dGQgY2xhc3M9ImIwMSBkYjEiPjIwPC90ZD48dGQgY2xhc3M9ImIwMiI%2BMTAzPC90ZD48dGQgY2xhc3M9ImIwMiI%2BMTg8L3RkPjx0ZCBjbGFzcz0iYjAyIHJlY3RfcmVkIj4yPC90ZD48dGQgY2xhc3M9ImIwMiI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjQ8L3RkPjx0ZCBjbGFzcz0iYjAyIj40MTwvdGQ%2BPHRyIGNsYXNzPSJkYjIiPg0KPHRkPjIwMTctMDItMDk8L3RkPjx0ZCBjbGFzcz0idDAyIj4xNzAyMDkwNjwvdGQ%2BPHRkIGNsYXNzPSJkYjEgZnciPjA2IDA0IDExIDAzIDAxPC90ZD48dGQgY2xhc3M9ImRiMSBmdyI%2BMDEgMDMgMDQgMDYgMTE8L3RkPjx0ZCBjbGFzcz0iYjA1IGJhbGxfcmVkIj4xPC90ZD48dGQgY2xhc3M9ImIwNSI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQiPjM8L3RkPjx0ZCBjbGFzcz0iYjA1IGJhbGxfcmVkIj40PC90ZD48dGQgY2xhc3M9ImIwNSI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQiPjY8L3RkPjx0ZCBjbGFzcz0iYjA1Ij40PC90ZD48dGQgY2xhc3M9ImIwNSI%2BMjwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjE8L3RkPjx0ZCBjbGFzcz0iYjA1Ij4xPC90ZD48dGQgY2xhc3M9ImIwNSBiYWxsX3JlZCBkYjEiPjExPC90ZD48dGQgY2xhc3M9ImIwMiI%2BMjA0ODwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjEyPC90ZD48dGQgY2xhc3M9ImIwMiI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDIgcmVjdF9ibHVlIj4zPC90ZD48dGQgY2xhc3M9ImIwMiI%2BNjwvdGQ%2BPHRkIGNsYXNzPSJiMDIgZGIxIj41PC90ZD48dGQgY2xhc3M9ImIwMSI%2BMzA3PC90ZD48dGQgY2xhc3M9ImIwMSI%2BMTc8L3RkPjx0ZCBjbGFzcz0iYjAxIHJlY3RfeWVsbG93Ij4yPC90ZD48dGQgY2xhc3M9ImIwMSI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDEiPjk8L3RkPjx0ZCBjbGFzcz0iYjAxIGRiMSI%2BMjE8L3RkPjx0ZCBjbGFzcz0iYjAyIj4xMDQ8L3RkPjx0ZCBjbGFzcz0iYjAyIj4xOTwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjE8L3RkPjx0ZCBjbGFzcz0iYjAyIHJlY3RfcmVkIj4zPC90ZD48dGQgY2xhc3M9ImIwMiI%2BNTwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjQyPC90ZD48dHI%2BDQo8dGQ%2BMjAxNy0wMi0wOTwvdGQ%2BPHRkIGNsYXNzPSJ0MDIiPjE3MDIwOTA3PC90ZD48dGQgY2xhc3M9ImRiMSBmdyI%2BMTEgMDMgMDcgMDkgMDQ8L3RkPjx0ZCBjbGFzcz0iZGIxIGZ3Ij4wMyAwNCAwNyAwOSAxMTwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjE8L3RkPjx0ZCBjbGFzcz0iYjA1Ij4yPC90ZD48dGQgY2xhc3M9ImIwNSBiYWxsX3JlZCI%2BMzwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQiPjQ8L3RkPjx0ZCBjbGFzcz0iYjA1Ij4yPC90ZD48dGQgY2xhc3M9ImIwNSI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQiPjc8L3RkPjx0ZCBjbGFzcz0iYjA1Ij4zPC90ZD48dGQgY2xhc3M9ImIwNSBiYWxsX3JlZCI%2BOTwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjI8L3RkPjx0ZCBjbGFzcz0iYjA1IGJhbGxfcmVkIGRiMSI%2BMTE8L3RkPjx0ZCBjbGFzcz0iYjAyIj4yMDQ5PC90ZD48dGQgY2xhc3M9ImIwMiI%2BMTM8L3RkPjx0ZCBjbGFzcz0iYjAyIj4yPC90ZD48dGQgY2xhc3M9ImIwMiI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDIgcmVjdF9ibHVlIj40PC90ZD48dGQgY2xhc3M9ImIwMiBkYjEiPjY8L3RkPjx0ZCBjbGFzcz0iYjAxIj4zMDg8L3RkPjx0ZCBjbGFzcz0iYjAxIj4xODwvdGQ%2BPHRkIGNsYXNzPSJiMDEiPjE8L3RkPjx0ZCBjbGFzcz0iYjAxIHJlY3RfeWVsbG93Ij4zPC90ZD48dGQgY2xhc3M9ImIwMSI%2BMTA8L3RkPjx0ZCBjbGFzcz0iYjAxIGRiMSI%2BMjI8L3RkPjx0ZCBjbGFzcz0iYjAyIj4xMDU8L3RkPjx0ZCBjbGFzcz0iYjAyIj4yMDwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjI8L3RkPjx0ZCBjbGFzcz0iYjAyIHJlY3RfcmVkIj4zPC90ZD48dGQgY2xhc3M9ImIwMiI%2BNjwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjQzPC90ZD48dHI%2BDQo8dGQ%2BMjAxNy0wMi0wOTwvdGQ%2BPHRkIGNsYXNzPSJ0MDIiPjE3MDIwOTA4PC90ZD48dGQgY2xhc3M9ImRiMSBmdyI%2BMDkgMDMgMDUgMTEgMTA8L3RkPjx0ZCBjbGFzcz0iZGIxIGZ3Ij4wMyAwNSAwOSAxMCAxMTwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjI8L3RkPjx0ZCBjbGFzcz0iYjA1Ij4zPC90ZD48dGQgY2xhc3M9ImIwNSBiYWxsX3JlZCI%2BMzwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjE8L3RkPjx0ZCBjbGFzcz0iYjA1IGJhbGxfcmVkIj41PC90ZD48dGQgY2xhc3M9ImIwNSI%2BMjwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjE8L3RkPjx0ZCBjbGFzcz0iYjA1Ij40PC90ZD48dGQgY2xhc3M9ImIwNSBiYWxsX3JlZCI%2BOTwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQiPjEwPC90ZD48dGQgY2xhc3M9ImIwNSBiYWxsX3JlZCBkYjEiPjExPC90ZD48dGQgY2xhc3M9ImIwMiI%2BMjA1MDwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjE0PC90ZD48dGQgY2xhc3M9ImIwMiI%2BMzwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjI8L3RkPjx0ZCBjbGFzcz0iYjAyIHJlY3RfYmx1ZSI%2BNDwvdGQ%2BPHRkIGNsYXNzPSJiMDIgZGIxIj43PC90ZD48dGQgY2xhc3M9ImIwMSI%2BMzA5PC90ZD48dGQgY2xhc3M9ImIwMSI%2BMTk8L3RkPjx0ZCBjbGFzcz0iYjAxIj4yPC90ZD48dGQgY2xhc3M9ImIwMSByZWN0X3llbGxvdyI%2BMzwvdGQ%2BPHRkIGNsYXNzPSJiMDEiPjExPC90ZD48dGQgY2xhc3M9ImIwMSBkYjEiPjIzPC90ZD48dGQgY2xhc3M9ImIwMiI%2BMTA2PC90ZD48dGQgY2xhc3M9ImIwMiI%2BMjE8L3RkPjx0ZCBjbGFzcz0iYjAyIj4zPC90ZD48dGQgY2xhc3M9ImIwMiByZWN0X3JlZCI%2BMzwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjc8L3RkPjx0ZCBjbGFzcz0iYjAyIj40NDwvdGQ%2BPHRyPg0KPHRkPjIwMTctMDItMDk8L3RkPjx0ZCBjbGFzcz0idDAyIj4xNzAyMDkwOTwvdGQ%2BPHRkIGNsYXNzPSJkYjEgZnciPjA3IDExIDEwIDAzIDA0PC90ZD48dGQgY2xhc3M9ImRiMSBmdyI%2BMDMgMDQgMDcgMTAgMTE8L3RkPjx0ZCBjbGFzcz0iYjA1Ij4zPC90ZD48dGQgY2xhc3M9ImIwNSI%2BNDwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQiPjM8L3RkPjx0ZCBjbGFzcz0iYjA1IGJhbGxfcmVkIj40PC90ZD48dGQgY2xhc3M9ImIwNSI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjM8L3RkPjx0ZCBjbGFzcz0iYjA1IGJhbGxfcmVkIj43PC90ZD48dGQgY2xhc3M9ImIwNSI%2BNTwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjE8L3RkPjx0ZCBjbGFzcz0iYjA1IGJhbGxfcmVkIj4xMDwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQgZGIxIj4xMTwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjIwNTE8L3RkPjx0ZCBjbGFzcz0iYjAyIj4xNTwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjQ8L3RkPjx0ZCBjbGFzcz0iYjAyIHJlY3RfYmx1ZSI%2BMzwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjE8L3RkPjx0ZCBjbGFzcz0iYjAyIGRiMSI%2BODwvdGQ%2BPHRkIGNsYXNzPSJiMDEiPjMxMDwvdGQ%2BPHRkIGNsYXNzPSJiMDEiPjIwPC90ZD48dGQgY2xhc3M9ImIwMSI%2BMzwvdGQ%2BPHRkIGNsYXNzPSJiMDEgcmVjdF95ZWxsb3ciPjM8L3RkPjx0ZCBjbGFzcz0iYjAxIj4xMjwvdGQ%2BPHRkIGNsYXNzPSJiMDEgZGIxIj4yNDwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjEwNzwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjIyPC90ZD48dGQgY2xhc3M9ImIwMiI%2BNDwvdGQ%2BPHRkIGNsYXNzPSJiMDIgcmVjdF9yZWQiPjM8L3RkPjx0ZCBjbGFzcz0iYjAyIj44PC90ZD48dGQgY2xhc3M9ImIwMiI%2BNDU8L3RkPjx0cj4NCjx0ZD4yMDE3LTAyLTA5PC90ZD48dGQgY2xhc3M9InQwMiI%2BMTcwMjA5MTA8L3RkPjx0ZCBjbGFzcz0iZGIxIGZ3Ij4xMSAwOCAwNSAwMyAxMDwvdGQ%2BPHRkIGNsYXNzPSJkYjEgZnciPjAzIDA1IDA4IDEwIDExPC90ZD48dGQgY2xhc3M9ImIwNSI%2BNDwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjU8L3RkPjx0ZCBjbGFzcz0iYjA1IGJhbGxfcmVkIj4zPC90ZD48dGQgY2xhc3M9ImIwNSI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQiPjU8L3RkPjx0ZCBjbGFzcz0iYjA1Ij40PC90ZD48dGQgY2xhc3M9ImIwNSI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQiPjg8L3RkPjx0ZCBjbGFzcz0iYjA1Ij4yPC90ZD48dGQgY2xhc3M9ImIwNSBiYWxsX3JlZCI%2BMTA8L3RkPjx0ZCBjbGFzcz0iYjA1IGJhbGxfcmVkIGRiMSI%2BMTE8L3RkPjx0ZCBjbGFzcz0iYjAyIj4yMDUyPC90ZD48dGQgY2xhc3M9ImIwMiI%2BMTY8L3RkPjx0ZCBjbGFzcz0iYjAyIj41PC90ZD48dGQgY2xhc3M9ImIwMiByZWN0X2JsdWUiPjM8L3RkPjx0ZCBjbGFzcz0iYjAyIj4yPC90ZD48dGQgY2xhc3M9ImIwMiBkYjEiPjk8L3RkPjx0ZCBjbGFzcz0iYjAxIj4zMTE8L3RkPjx0ZCBjbGFzcz0iYjAxIj4yMTwvdGQ%2BPHRkIGNsYXNzPSJiMDEiPjQ8L3RkPjx0ZCBjbGFzcz0iYjAxIHJlY3RfeWVsbG93Ij4zPC90ZD48dGQgY2xhc3M9ImIwMSI%2BMTM8L3RkPjx0ZCBjbGFzcz0iYjAxIGRiMSI%2BMjU8L3RkPjx0ZCBjbGFzcz0iYjAyIj4xMDg8L3RkPjx0ZCBjbGFzcz0iYjAyIj4yMzwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjU8L3RkPjx0ZCBjbGFzcz0iYjAyIHJlY3RfcmVkIj4zPC90ZD48dGQgY2xhc3M9ImIwMiI%2BOTwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjQ2PC90ZD48dHIgY2xhc3M9ImRiMiI%2BDQo8dGQ%2BMjAxNy0wMi0wOTwvdGQ%2BPHRkIGNsYXNzPSJ0MDIiPjE3MDIwOTExPC90ZD48dGQgY2xhc3M9ImRiMSBmdyI%2BMDYgMDEgMTEgMDkgMTA8L3RkPjx0ZCBjbGFzcz0iZGIxIGZ3Ij4wMSAwNiAwOSAxMCAxMTwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQiPjE8L3RkPjx0ZCBjbGFzcz0iYjA1Ij42PC90ZD48dGQgY2xhc3M9ImIwNSI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjI8L3RkPjx0ZCBjbGFzcz0iYjA1Ij4xPC90ZD48dGQgY2xhc3M9ImIwNSBiYWxsX3JlZCI%2BNjwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjI8L3RkPjx0ZCBjbGFzcz0iYjA1Ij4xPC90ZD48dGQgY2xhc3M9ImIwNSBiYWxsX3JlZCI%2BOTwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQiPjEwPC90ZD48dGQgY2xhc3M9ImIwNSBiYWxsX3JlZCBkYjEiPjExPC90ZD48dGQgY2xhc3M9ImIwMiI%2BMjA1MzwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjE3PC90ZD48dGQgY2xhc3M9ImIwMiI%2BNjwvdGQ%2BPHRkIGNsYXNzPSJiMDIgcmVjdF9ibHVlIj4zPC90ZD48dGQgY2xhc3M9ImIwMiI%2BMzwvdGQ%2BPHRkIGNsYXNzPSJiMDIgZGIxIj4xMDwvdGQ%2BPHRkIGNsYXNzPSJiMDEiPjMxMjwvdGQ%2BPHRkIGNsYXNzPSJiMDEiPjIyPC90ZD48dGQgY2xhc3M9ImIwMSI%2BNTwvdGQ%2BPHRkIGNsYXNzPSJiMDEiPjE8L3RkPjx0ZCBjbGFzcz0iYjAxIHJlY3RfeWVsbG93Ij40PC90ZD48dGQgY2xhc3M9ImIwMSBkYjEiPjI2PC90ZD48dGQgY2xhc3M9ImIwMiI%2BMTA5PC90ZD48dGQgY2xhc3M9ImIwMiI%2BMjQ8L3RkPjx0ZCBjbGFzcz0iYjAyIHJlY3RfcmVkIj4yPC90ZD48dGQgY2xhc3M9ImIwMiI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjEwPC90ZD48dGQgY2xhc3M9ImIwMiI%2BNDc8L3RkPjx0cj4NCjx0ZD4yMDE3LTAyLTA5PC90ZD48dGQgY2xhc3M9InQwMiI%2BMTcwMjA5MTI8L3RkPjx0ZCBjbGFzcz0iZGIxIGZ3Ij4wNCAwMSAxMSAwNyAwODwvdGQ%2BPHRkIGNsYXNzPSJkYjEgZnciPjAxIDA0IDA3IDA4IDExPC90ZD48dGQgY2xhc3M9ImIwNSBiYWxsX3JlZCI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjc8L3RkPjx0ZCBjbGFzcz0iYjA1Ij4yPC90ZD48dGQgY2xhc3M9ImIwNSBiYWxsX3JlZCI%2BNDwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjI8L3RkPjx0ZCBjbGFzcz0iYjA1Ij4xPC90ZD48dGQgY2xhc3M9ImIwNSBiYWxsX3JlZCI%2BNzwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQiPjg8L3RkPjx0ZCBjbGFzcz0iYjA1Ij4xPC90ZD48dGQgY2xhc3M9ImIwNSI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQgZGIxIj4xMTwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjIwNTQ8L3RkPjx0ZCBjbGFzcz0iYjAyIj4xODwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjc8L3RkPjx0ZCBjbGFzcz0iYjAyIHJlY3RfYmx1ZSI%2BMzwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjQ8L3RkPjx0ZCBjbGFzcz0iYjAyIGRiMSI%2BMTE8L3RkPjx0ZCBjbGFzcz0iYjAxIj4zMTM8L3RkPjx0ZCBjbGFzcz0iYjAxIj4yMzwvdGQ%2BPHRkIGNsYXNzPSJiMDEiPjY8L3RkPjx0ZCBjbGFzcz0iYjAxIHJlY3RfeWVsbG93Ij4zPC90ZD48dGQgY2xhc3M9ImIwMSI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDEgZGIxIj4yNzwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjExMDwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjI1PC90ZD48dGQgY2xhc3M9ImIwMiI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDIgcmVjdF9yZWQiPjM8L3RkPjx0ZCBjbGFzcz0iYjAyIj4xMTwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjQ4PC90ZD48dHI%2BDQo8dGQ%2BMjAxNy0wMi0wOTwvdGQ%2BPHRkIGNsYXNzPSJ0MDIiPjE3MDIwOTEzPC90ZD48dGQgY2xhc3M9ImRiMSBmdyI%2BMTEgMDUgMDIgMDYgMDQ8L3RkPjx0ZCBjbGFzcz0iZGIxIGZ3Ij4wMiAwNCAwNSAwNiAxMTwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjE8L3RkPjx0ZCBjbGFzcz0iYjA1IGJhbGxfcmVkIj4yPC90ZD48dGQgY2xhc3M9ImIwNSI%2BMzwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQiPjQ8L3RkPjx0ZCBjbGFzcz0iYjA1IGJhbGxfcmVkIj41PC90ZD48dGQgY2xhc3M9ImIwNSBiYWxsX3JlZCI%2BNjwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjE8L3RkPjx0ZCBjbGFzcz0iYjA1Ij4xPC90ZD48dGQgY2xhc3M9ImIwNSI%2BMjwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjI8L3RkPjx0ZCBjbGFzcz0iYjA1IGJhbGxfcmVkIGRiMSI%2BMTE8L3RkPjx0ZCBjbGFzcz0iYjAyIj4yMDU1PC90ZD48dGQgY2xhc3M9ImIwMiI%2BMTk8L3RkPjx0ZCBjbGFzcz0iYjAyIHJlY3RfYmx1ZSI%2BMjwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjE8L3RkPjx0ZCBjbGFzcz0iYjAyIj41PC90ZD48dGQgY2xhc3M9ImIwMiBkYjEiPjEyPC90ZD48dGQgY2xhc3M9ImIwMSI%2BMzE0PC90ZD48dGQgY2xhc3M9ImIwMSI%2BMjQ8L3RkPjx0ZCBjbGFzcz0iYjAxIHJlY3RfeWVsbG93Ij4yPC90ZD48dGQgY2xhc3M9ImIwMSI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDEiPjI8L3RkPjx0ZCBjbGFzcz0iYjAxIGRiMSI%2BMjg8L3RkPjx0ZCBjbGFzcz0iYjAyIj4xMTE8L3RkPjx0ZCBjbGFzcz0iYjAyIj4yNjwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjI8L3RkPjx0ZCBjbGFzcz0iYjAyIHJlY3RfcmVkIj4zPC90ZD48dGQgY2xhc3M9ImIwMiI%2BMTI8L3RkPjx0ZCBjbGFzcz0iYjAyIj40OTwvdGQ%2BPHRyPg0KPHRkPjIwMTctMDItMDk8L3RkPjx0ZCBjbGFzcz0idDAyIj4xNzAyMDkxNDwvdGQ%2BPHRkIGNsYXNzPSJkYjEgZnciPjA4IDExIDAzIDA3IDEwPC90ZD48dGQgY2xhc3M9ImRiMSBmdyI%2BMDMgMDcgMDggMTAgMTE8L3RkPjx0ZCBjbGFzcz0iYjA1Ij4yPC90ZD48dGQgY2xhc3M9ImIwNSI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQiPjM8L3RkPjx0ZCBjbGFzcz0iYjA1Ij4xPC90ZD48dGQgY2xhc3M9ImIwNSI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjE8L3RkPjx0ZCBjbGFzcz0iYjA1IGJhbGxfcmVkIj43PC90ZD48dGQgY2xhc3M9ImIwNSBiYWxsX3JlZCI%2BODwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjM8L3RkPjx0ZCBjbGFzcz0iYjA1IGJhbGxfcmVkIj4xMDwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQgZGIxIj4xMTwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjIwNTY8L3RkPjx0ZCBjbGFzcz0iYjAyIj4yMDwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjE8L3RkPjx0ZCBjbGFzcz0iYjAyIHJlY3RfYmx1ZSI%2BMzwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjY8L3RkPjx0ZCBjbGFzcz0iYjAyIGRiMSI%2BMTM8L3RkPjx0ZCBjbGFzcz0iYjAxIj4zMTU8L3RkPjx0ZCBjbGFzcz0iYjAxIj4yNTwvdGQ%2BPHRkIGNsYXNzPSJiMDEiPjE8L3RkPjx0ZCBjbGFzcz0iYjAxIj4yPC90ZD48dGQgY2xhc3M9ImIwMSByZWN0X3llbGxvdyI%2BNDwvdGQ%2BPHRkIGNsYXNzPSJiMDEgZGIxIj4yOTwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjExMjwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjI3PC90ZD48dGQgY2xhc3M9ImIwMiI%2BMzwvdGQ%2BPHRkIGNsYXNzPSJiMDIgcmVjdF9yZWQiPjM8L3RkPjx0ZCBjbGFzcz0iYjAyIj4xMzwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjUwPC90ZD48dHI%2BDQo8dGQ%2BMjAxNy0wMi0wOTwvdGQ%2BPHRkIGNsYXNzPSJ0MDIiPjE3MDIwOTE1PC90ZD48dGQgY2xhc3M9ImRiMSBmdyI%2BMDYgMDIgMDQgMTEgMDg8L3RkPjx0ZCBjbGFzcz0iZGIxIGZ3Ij4wMiAwNCAwNiAwOCAxMTwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjM8L3RkPjx0ZCBjbGFzcz0iYjA1IGJhbGxfcmVkIj4yPC90ZD48dGQgY2xhc3M9ImIwNSI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQiPjQ8L3RkPjx0ZCBjbGFzcz0iYjA1Ij4yPC90ZD48dGQgY2xhc3M9ImIwNSBiYWxsX3JlZCI%2BNjwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjE8L3RkPjx0ZCBjbGFzcz0iYjA1IGJhbGxfcmVkIj44PC90ZD48dGQgY2xhc3M9ImIwNSI%2BNDwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjE8L3RkPjx0ZCBjbGFzcz0iYjA1IGJhbGxfcmVkIGRiMSI%2BMTE8L3RkPjx0ZCBjbGFzcz0iYjAyIj4yMDU3PC90ZD48dGQgY2xhc3M9ImIwMiByZWN0X2JsdWUiPjE8L3RkPjx0ZCBjbGFzcz0iYjAyIj4yPC90ZD48dGQgY2xhc3M9ImIwMiI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjc8L3RkPjx0ZCBjbGFzcz0iYjAyIGRiMSI%2BMTQ8L3RkPjx0ZCBjbGFzcz0iYjAxIj4zMTY8L3RkPjx0ZCBjbGFzcz0iYjAxIj4yNjwvdGQ%2BPHRkIGNsYXNzPSJiMDEiPjI8L3RkPjx0ZCBjbGFzcz0iYjAxIHJlY3RfeWVsbG93Ij4zPC90ZD48dGQgY2xhc3M9ImIwMSI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDEgZGIxIj4zMDwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjExMzwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjI4PC90ZD48dGQgY2xhc3M9ImIwMiByZWN0X3JlZCI%2BMjwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjE8L3RkPjx0ZCBjbGFzcz0iYjAyIj4xNDwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjUxPC90ZD48dHIgY2xhc3M9ImRiMiI%2BDQo8dGQ%2BMjAxNy0wMi0wOTwvdGQ%2BPHRkIGNsYXNzPSJ0MDIiPjE3MDIwOTE2PC90ZD48dGQgY2xhc3M9ImRiMSBmdyI%2BMDEgMDggMDkgMDIgMDc8L3RkPjx0ZCBjbGFzcz0iZGIxIGZ3Ij4wMSAwMiAwNyAwOCAwOTwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQiPjE8L3RkPjx0ZCBjbGFzcz0iYjA1IGJhbGxfcmVkIj4yPC90ZD48dGQgY2xhc3M9ImIwNSI%2BMjwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjE8L3RkPjx0ZCBjbGFzcz0iYjA1Ij4zPC90ZD48dGQgY2xhc3M9ImIwNSI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDUgYmFsbF9yZWQiPjc8L3RkPjx0ZCBjbGFzcz0iYjA1IGJhbGxfcmVkIj44PC90ZD48dGQgY2xhc3M9ImIwNSBiYWxsX3JlZCI%2BOTwvdGQ%2BPHRkIGNsYXNzPSJiMDUiPjI8L3RkPjx0ZCBjbGFzcz0iYjA1IGRiMSI%2BMTwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjIwNTg8L3RkPjx0ZCBjbGFzcz0iYjAyIj4xPC90ZD48dGQgY2xhc3M9ImIwMiI%2BMzwvdGQ%2BPHRkIGNsYXNzPSJiMDIgcmVjdF9ibHVlIj4zPC90ZD48dGQgY2xhc3M9ImIwMiI%2BODwvdGQ%2BPHRkIGNsYXNzPSJiMDIgZGIxIj4xNTwvdGQ%2BPHRkIGNsYXNzPSJiMDEiPjMxNzwvdGQ%2BPHRkIGNsYXNzPSJiMDEiPjI3PC90ZD48dGQgY2xhc3M9ImIwMSI%2BMzwvdGQ%2BPHRkIGNsYXNzPSJiMDEgcmVjdF95ZWxsb3ciPjM8L3RkPjx0ZCBjbGFzcz0iYjAxIj4yPC90ZD48dGQgY2xhc3M9ImIwMSBkYjEiPjMxPC90ZD48dGQgY2xhc3M9ImIwMiI%2BMTE0PC90ZD48dGQgY2xhc3M9ImIwMiI%2BMjk8L3RkPjx0ZCBjbGFzcz0iYjAyIj4xPC90ZD48dGQgY2xhc3M9ImIwMiByZWN0X3JlZCI%2BMzwvdGQ%2BPHRkIGNsYXNzPSJiMDIiPjE1PC90ZD48dGQgY2xhc3M9ImIwMiI%2BNTI8L3RkPjwvdGFibGU%2BDQpkZPtGCk0%2B0mcn0sGOHEkYuvybwud5&__VIEWSTATEGENERATOR=578D3B3C&__EVENTVALIDATION=%2FwEWCAKBxNuBAgKQ%2FOqYBwKAk8D2CwKfk8D2CwKek8D2CwKdk8D2CwKck8D2CwKbk8D2CzXqzsrQF3tk2yuuWiA97yk04MOu&drop1=2";
            return GetOpenListFromMainUrl(yesterdaypostdata);
        }

        /// <summary>
        /// 抓取主站点开奖数据
        /// </summary>
        /// <param name="postdata">传送数据</param>
        /// <returns></returns>
        private Dictionary<string, string> GetOpenListFromMainUrl(string postdata)
        {
            Dictionary<string, string> result = new Dictionary<string, string>();
            try
            {
                var HtmlResource = NetHelper.GetUrlResponse(Config.MainUrl, "POST", postdata);
                if (string.IsNullOrWhiteSpace(HtmlResource)) return result;
                Regex reg = new Regex(@"<td class=""t02"">(.*?)</td><td class=""db1 fw"">(.*?)</td>", RegexOptions.IgnoreCase);
                MatchCollection mc = reg.Matches(HtmlResource);
                string matchQiHao = string.Empty;
                string matchKJHaoMa = string.Empty;
                foreach (Match m in mc)
                {
                    if (!m.Success) continue;
                    matchQiHao = m.Result("$1").Trim();
                    matchKJHaoMa = m.Result("$2").Trim().Replace(" ", ",");
                    result.Add(matchQiHao, matchKJHaoMa);
                }
            }
            catch (Exception ex)
            {
                log.Error(typeof(YN11X5Job), string.Format("【{0}】通过主站点抓取开奖列表时发生错误，错误信息【{1}】", Config.Area + Config.LotteryName, ex.Message));
            }
            return result;
        }

        /// <summary>
        /// 通过备用站点抓取开奖数据
        /// (爱彩乐)
        /// </summary>
        private void DoTodayJobByBackUrl()
        {
            if (!string.IsNullOrEmpty(Config.BackUrl))
            {
                var OpenList = GetTodayOpenListFromBackUrl();
                if (OpenList.Count == 0) return;//无抓取数据
                var newestQiHao = OpenList.First().Key;
                var startQiNum = Convert.ToInt32(LatestQiHao.Substring(6)) + 1;
                var newestQiNum = Convert.ToInt32(newestQiHao.Substring(6));
                if (startQiNum > newestQiNum) return;//无最新数据
                //处理最新开奖数据
                var getQiHao = string.Empty;
                for (var i = startQiNum; i <= newestQiNum; i++)
                {
                    getQiHao = CommonHelper.GenerateTodayQiHaoYYMMDDQQ(i);
                    var matchItem = OpenList.Where(R => R.Key == getQiHao).FirstOrDefault();
                    if (matchItem.Key != null && SaveRecord(getQiHao, matchItem.Value, false))
                    {
                        //处理成功写入日志
                        log.Info(typeof(YN11X5Job), CommonHelper.GetJobBackLogInfo(Config, getQiHao));
                        LatestQiHao = getQiHao;
                    }
                }
            }
        }

        /// <summary>
        /// 通过备用地址抓取错误期号列表中每一个期号
        /// (爱彩乐)
        /// </summary>
        private void DoYesterdayFailedListByBackUrl()
        {
            if (!string.IsNullOrEmpty(Config.BackUrl) && FailedQiHaoList.Count > 0)
            {
                var OpenList = GetYesterdayOpenListFromBackUrl();
                if (OpenList.Count == 0) return;//无抓取数据
                var SuccessList = new List<string>();
                foreach (var failedQiHao in FailedQiHaoList)
                {
                    var matchItem = OpenList.Where(R => R.Key == failedQiHao).FirstOrDefault();
                    if (matchItem.Key != null && SaveRecord(failedQiHao, matchItem.Value, true))
                    {
                        //处理成功写入日志
                        log.Info(typeof(YN11X5Job), CommonHelper.GetJobBackLogInfo(Config, failedQiHao));
                        SuccessList.Add(failedQiHao);
                        continue;
                    }
                }
                foreach (var successQiHao in SuccessList)
                {
                    FailedQiHaoList.Remove(successQiHao);
                }
            }
        }
        /// <summary>
        /// 抓取备用站点开奖数据
        /// </summary>
        /// <returns></returns>
        private Dictionary<string, string> GetTodayOpenListFromBackUrl()
        {
            Dictionary<string, string> result = new Dictionary<string, string>();
            try
            {
                var HtmlResource = NetHelper.GetUrlResponse(Config.BackUrl);
                if (!string.IsNullOrWhiteSpace(HtmlResource))
                {
                    HtmlDocument doc = new HtmlDocument();
                    doc.LoadHtml(HtmlResource);
                    var table = doc.DocumentNode.SelectSingleNode("//table");
                    if (table == null) return result;
                    var trs = table.ChildNodes.Where(R => R.Name.ToLower() == "tr").ToList();
                    List<HtmlNode> tds = null, ems = null;
                    var matchQiHao = string.Empty;
                    var matchKJHaoMa = string.Empty;
                    for (var i = 1; i < trs.Count; i++)//第一行为表头
                    {
                        tds = trs[i].ChildNodes.Where(R => R.Name.ToLower() == "td").ToList();
                        if (tds.Count < 3) continue;
                        matchQiHao = tds[0].InnerText.Trim();
                        ems = tds[2].ChildNodes.Where(R => R.Name.ToLower() == "em").ToList();
                        if (ems.Count < 5) continue;
                        matchKJHaoMa = string.Format("{0},{1},{2},{3},{4}", ems[0].InnerText.Trim(), ems[1].InnerText.Trim(), ems[2].InnerText.Trim(), ems[3].InnerText.Trim(), ems[4].InnerText.Trim());
                        if (!result.ContainsKey(matchQiHao))
                            result.Add(matchQiHao, matchKJHaoMa);
                    }
                }
            }
            catch (Exception ex)
            {
                log.Error(typeof(YN11X5Job), string.Format("【{0}】通过备用站点抓取今日最新开奖列表时发生错误，错误信息【{1}】", Config.Area + Config.LotteryName, ex.Message));
            }
            return result;
        }
        /// <summary>
        /// 通过备用站点抓取昨日开奖列表
        /// </summary>
        /// <returns></returns>
        private Dictionary<string, string> GetYesterdayOpenListFromBackUrl()
        {
            Dictionary<string, string> result = new Dictionary<string, string>();
            try
            {
                var HtmlResource = NetHelper.GetUrlResponse(Config.BackUrl + "?action=chart&date=yesterday&id=519&async=true");
                if (!string.IsNullOrWhiteSpace(HtmlResource))
                {
                    var obj = JsonConvert.DeserializeObject<JObject>(HtmlResource);
                    if (obj != null && obj["data"] != null)
                    {
                        var matchQiHao = string.Empty;
                        var matchKJHaoMa = string.Empty;
                        JArray openCodeList = null;
                        foreach (var item in obj["data"])
                        {
                            matchQiHao = item["dateNumber"].ToString();
                            openCodeList = (JArray)item["list"];
                            matchKJHaoMa = string.Format("{0},{1},{2},{3},{4}", openCodeList[0].ToString(), openCodeList[1].ToString(), openCodeList[2].ToString(), openCodeList[3].ToString(), openCodeList[4].ToString());
                            if (!result.ContainsKey(matchQiHao))
                                result.Add(matchQiHao, matchKJHaoMa);
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                log.Error(typeof(YN11X5Job), string.Format("【{0}】通过备用站点抓取昨日开奖列表时发生错误，错误信息【{1}】", Config.Area + Config.LotteryName, ex.Message));
            }
            return result;
        }

        /// <summary>
        /// 将此彩种指定期号和开奖号码保存至数据库
        /// </summary>
        /// <param name="QiHao">期号</param>
        /// <param name="OpenCode">开奖号码(形如01,02,03,04,05)</param>
        /// <param name="IsYesterdayRecord">是否是保存昨天的记录</param>
        /// <returns></returns>
        private bool SaveRecord(string QiHao, string OpenCode, bool IsYesterdayRecord)
        {
            if (!string.IsNullOrWhiteSpace(QiHao) && !string.IsNullOrWhiteSpace(OpenCode))
            {
                OpenCode5Model model = new OpenCode5Model();
                model.Term = Convert.ToInt64(QiHao);
                var haoMaArray = OpenCode.Split(new char[] { ',' });
                model.OpenCode1 = Convert.ToInt32(haoMaArray[0]);
                model.OpenCode2 = Convert.ToInt32(haoMaArray[1]);
                model.OpenCode3 = Convert.ToInt32(haoMaArray[2]);
                model.OpenCode4 = Convert.ToInt32(haoMaArray[3]);
                model.OpenCode5 = Convert.ToInt32(haoMaArray[4]);
                if (IsYesterdayRecord)
                    model.OpenTime = CommonHelper.GenerateYesterdayOpenTime(Config, QiHao);
                else
                    model.OpenTime = CommonHelper.GenerateTodayOpenTime(Config, QiHao);
                return services.AddOpen5Code(currentLottery, model);
            }
            return false;
        }

        #region Attribute
        /// <summary>
        /// 配置信息
        /// </summary>
        private SCCConfig Config = null;
        /// <summary>
        /// 当天抓取的最新一期期号
        /// </summary>
        private string LatestQiHao = null;
        /// <summary>
        /// 当天抓取失败列表
        /// </summary>
        private List<string> FailedQiHaoList = null;
        /// <summary>
        /// 日志对象
        /// </summary>
        private LogHelper log = null;
        /// <summary>
        /// 数据服务
        /// </summary>
        private IOpen5Code services = null;
        /// <summary>
        /// 当前彩种
        /// </summary>
        private SCCLottery currentLottery
        {
            get
            {
                return SCCLottery.YunNan11X5;
            }
        }
        /// <summary>
        /// 邮件接口
        /// </summary>
        private IEmail email = null;
        #endregion
    }
}
